local brain1 = require "brains/zg_deerclopsbrain"
local brain2 = require "brains/zg_deerclopsbrain2"

local assets =
{
    Asset("ANIM", "anim/deerclops_basic.zip"),
    --Asset("ANIM", "anim/deerclops_actions.zip"),
    --Asset("ANIM", "anim/deerclops_build.zip"),
    --Asset("ANIM", "anim/deerclops_neutral_build.zip"),
    Asset("SOUND", "sound/deerclops.fsb"),	
}

local prefabs =
{
    "meat",
    "deerclops_eyeball",
    "icespike_fx_1",
    "icespike_fx_2",
    "icespike_fx_3",
    "icespike_fx_4",
}

local TARGET_DIST = 48
local STRUCTURES_PER_HARASS = 5

local function IsSated(inst)
    return inst.structuresDestroyed >= STRUCTURES_PER_HARASS
end

local function WantsToLeave(inst)
    return not inst.components.combat:HasTarget()
    and inst:IsSated()
    and inst:GetTimeAlive() >= 120
end

local function CalcSanityAura(inst)
    return inst.components.combat.target ~= nil and -TUNING.SANITYAURA_HUGE or -TUNING.SANITYAURA_LARGE
end

local function FindBaseToAttack(inst, target)
    --print("Deerclops finding base to attack")
    local structure = GetClosestInstWithTag("structure", target, 40)
    if structure then
        local targetPos = Vector3(structure.Transform:GetWorldPosition() )
        inst.components.knownlocations:RememberLocation("targetbase", targetPos)
        inst.AnimState:ClearOverrideSymbol("deerclops_head")
    end
end

local function RetargetFn(inst)
	local invader = nil
    invader = FindEntity(inst, TARGET_DIST, function(guy)
        return guy:HasTag("player") and not guy:HasTag("shadowboss") and not guy:HasTag("playerghost")
    end)
	if invader then
		if inst:GetPosition():Dist(invader:GetPosition()) < 3 then
			inst.components.combat:SetTarget(invader)
		end
	end
	return invader
end

local function KeepTargetFn(inst, target)
    return inst.components.combat:CanTarget(target)
end

local function AfterWorking(inst, data)
    if data.target then
        local recipe = AllRecipes[data.target.prefab]
        if recipe then
            inst.structuresDestroyed = inst.structuresDestroyed + 1
            if inst:IsSated() then
                inst.components.knownlocations:ForgetLocation("targetbase")
                inst.AnimState:OverrideSymbol("deerclops_head", "deerclops_neutral_build", "deerclops_head")
            end
        end
    end
end

local function ShouldSleep(inst)
    return false
end

local function ShouldWake(inst)
    return true
end

local function OnEntitySleep(inst)
    if inst:WantsToLeave() then
        inst.structuresDestroyed = 0 -- reset this for the stored version
        TheWorld:PushEvent("storehassler", inst)
        inst:Remove()
    end
end

local function OnStopWinter(inst)
    if inst:IsAsleep() then
        TheWorld:PushEvent("storehassler", inst)
        inst:Remove()
    end
end

local function OnSave(inst, data)
    data.structuresDestroyed = inst.structuresDestroyed
end
        
local function OnLoad(inst, data)
    if data then
        inst.structuresDestroyed = data.structuresDestroyed or inst.structuresDestroyed
    end
end

local function OnAttacked(inst, data)
	local invader = nil
	invader = FindEntity(inst, 3, function(guy)
        return guy:HasTag("player") and not guy:HasTag("playerghost")
    end)
	if invader then
		inst.components.combat:SetTarget(invader)
	elseif data.attacker and not data.attacker:HasTag("shadowboss") then
		inst.components.combat:SetTarget(data.attacker)
	end
	
end

local function OnHitOther(inst, data)
    local other = data.target
    if other and other.components.freezable then
        other.components.freezable:AddColdness(2)
        other.components.freezable:SpawnShatterFX()
    end
end

local function OnRemove(inst)
    TheWorld:PushEvent("hasslerremoved", inst)
end

local function OnDead(inst)
    TheWorld:PushEvent("hasslerkilled", inst)
end

local function oncollapse(inst, other)
    if other:IsValid() and other.components.workable ~= nil and other.components.workable:CanBeWorked() then
        SpawnPrefab("collapse_small").Transform:SetPosition(other.Transform:GetWorldPosition())
        --other.components.workable:Destroy(inst)
		other.components.workable:WorkedBy(inst, 1)
    end
end

local function oncollide(inst, other)
    if other ~= nil and
        (other:HasTag("tree") or other:HasTag("boulder")) and --HasTag implies IsValid
        Vector3(inst.Physics:GetVelocity()):LengthSq() >= 1 then
        inst:DoTaskInTime(2 * FRAMES, oncollapse, other)
    end
end

local function OnNewTarget(inst, data)
    FindBaseToAttack(inst, data.target or inst)
    if inst.components.knownlocations:GetLocation("targetbase") and data.target:HasTag("player") then
        inst.structuresDestroyed = inst.structuresDestroyed - 1
        inst.components.knownlocations:ForgetLocation("home")
    end
end

local loot = {"meat", "meat", "meat", "meat", "meat", "meat", "meat", "meat", "deerclops_eyeball",
	"meat", "meat", "meat", "meat", "meat", "meat", "meat", "meat", "deerclops_eyeball",
}

local function ontimerdone(inst, data)
	if data.name == "icetime" then
		inst.canicetime = true
	elseif data.name == "laser"then 
		inst.canlaser = true
		inst.components.combat:SetRange(15, 5.5)
	end
end

local function fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddDynamicShadow()
	inst.entity:AddLight()
    inst.entity:AddNetwork()

    MakeGiantCharacterPhysics(inst, 1000, .5)

    local s  = 1.65
    inst.Transform:SetScale(s, s, s)
    inst.DynamicShadow:SetSize(6, 3.5)
    inst.Transform:SetFourFaced()
	
	--
	inst:AddTag("zg_epic_ch1")
	inst:AddTag("zg_marine")
	--
	inst:AddTag("zg_highhealth")
	inst:AddTag("zg_highattack")
	inst:AddTag("zg_hasabsorb")
	
    inst:AddTag("epic")
    inst:AddTag("monster")
    inst:AddTag("hostile")
    inst:AddTag("deerclops")
    inst:AddTag("scarytoprey")
    inst:AddTag("largecreature")
	
	inst:AddTag("shadowboss")		--专属标签

    inst.AnimState:SetBank("deerclops")
    inst.AnimState:SetBuild("zg_deerclops_build")
    inst.AnimState:PlayAnimation("idle_loop", true)
	inst.AnimState:OverrideSymbol("laserring", "zg_laserring", "zg_laserring")
	inst.AnimState:OverrideSymbol("laserstar", "zg_laserstar", "zg_laserstar")
	inst.AnimState:OverrideSymbol("snowflower", "zg_snowflower", "zg_snowflower")
	inst.AnimState:OverrideSymbol("snowflowersecond", "zg_snowflower", "zg_snowflower")
	
	inst.Light:Enable(false)
    inst.Light:SetIntensity(.9)
    inst.Light:SetColour(255 / 255, 255 / 255, 180 / 255)
    inst.Light:SetFalloff(0.3)
    inst.Light:SetRadius(12)

    inst.entity:SetPristine()

    if not TheWorld.ismastersim then
        return inst
    end

    inst.Physics:SetCollisionCallback(oncollide)

    inst.structuresDestroyed = 0

    ------------------------------------------
	inst:AddComponent("inventory")

    inst:AddComponent("locomotor") -- locomotor must be constructed before the stategraph
    inst.components.locomotor.walkspeed = 3  

    ------------------------------------------
    --inst:SetStateGraph("SGdeerclops")
	inst:SetStateGraph("zg_deerclops")
    ------------------------------------------

    inst:AddComponent("sanityaura")
    inst.components.sanityaura.aurafn = CalcSanityAura

    MakeLargeBurnableCharacter(inst, "deerclops_body")
    --MakeHugeFreezableCharacter(inst, "deerclops_body")

    ------------------
    inst:AddComponent("health")
    inst.components.health:SetMaxHealth(6000)
	inst.components.health.destroytime = 50*FRAMES
	inst.components.health.erodetime = 5*FRAMES
    ------------------

    inst:AddComponent("combat")
    inst.components.combat:SetDefaultDamage(75)
    --inst.components.combat.playerdamagepercent = 0.5
    inst.components.combat:SetRange(5.5)
    --inst.components.combat:SetAreaDamage(TUNING.DEERCLOPS_AOE_RANGE, TUNING.DEERCLOPS_AOE_SCALE)
    inst.components.combat.hiteffectsymbol = "deerclops_body"
    inst.components.combat:SetAttackPeriod(4)
    inst.components.combat:SetRetargetFunction(2, RetargetFn)
    inst.components.combat:SetKeepTargetFunction(KeepTargetFn)
	
	--[[
	function inst.components.combat:CanBeAttacked(attacker)
		return attacker
			and not attacker:HasTag("zg_marine")
			and self.inst.replica.combat:CanBeAttacked(attacker)
	end]]
	
	inst:AddComponent("timer")
	inst:ListenForEvent("timerdone", ontimerdone)
    ------------------------------------------

    inst:AddComponent("sleeper")
    inst.components.sleeper:SetResistance(4)
    inst.components.sleeper:SetSleepTest(ShouldSleep)
    inst.components.sleeper:SetWakeTest(ShouldWake)

    ------------------------------------------

    inst:AddComponent("lootdropper")
    inst.components.lootdropper:SetLoot(loot)

    ------------------------------------------

    inst:AddComponent("inspectable")
    inst.components.inspectable:RecordViews()
    ------------------------------------------
    inst:AddComponent("knownlocations")
	
    inst:SetBrain(brain2)
	
	--[[
	inst.zg_ai = 2
	----周期性换AI
	inst:DoPeriodicTask(20, function()
		if inst.zg_ai == 2 then 
			inst:SetBrain(brain2) inst.components.combat:SetAttackPeriod(4)
			inst.zg_ai = 1
			return
		elseif inst.zg_ai == 1 then
			inst:SetBrain(brain1) inst.components.combat:SetAttackPeriod(5)
			inst.zg_ai =2
			return
		end
	end)]]

    --inst:ListenForEvent("working", AfterWorking)
    --inst:ListenForEvent("entitysleep", OnEntitySleep)
    inst:ListenForEvent("attacked", OnAttacked)
    --inst:ListenForEvent("onhitother", OnHitOther)
    
	--inst:ListenForEvent("death", OnDead)
    --inst:ListenForEvent("onremove", OnRemove)
    --inst:ListenForEvent("newcombattarget", OnNewTarget)

    --inst:WatchWorldState("stopwinter", OnStopWinter)

    --inst.OnSave = OnSave
    --inst.OnLoad = OnLoad
    --inst.IsSated = IsSated
    --inst.WantsToLeave = WantsToLeave

    return inst
end

local function fn_laserfx()
	local inst = CreateEntity()
	
	inst.entity:AddTransform()
	inst.Transform:SetScale(2, 2, 2)
	
	inst.entity:AddSoundEmitter()
	inst.entity:AddLight()
	inst.entity:AddNetwork()
	
	inst.entity:AddAnimState()
	
	--
	inst.AnimState:SetBank("zg_laserfx")
	inst.AnimState:SetBuild("zg_laserfx")
	inst.AnimState:PlayAnimation("idle")
	inst.AnimState:SetOrientation(ANIM_ORIENTATION.OnGround)
    inst.AnimState:SetLayer(LAYER_BACKGROUND)
    inst.AnimState:SetSortOrder(3)	
	
	--
	inst:AddTag("FX")
	
	inst.Light:Enable(true)
	inst.Light:SetRadius(12)
	inst.Light:SetFalloff(.3)
	inst.Light:SetIntensity(0.9)
	inst.Light:SetColour(255/255,255/255,180/255)
	
	inst.entity:SetPristine()

    if not TheWorld.ismastersim then
       return inst
    end
	
	--inst.Light:Enable(true)
    --inst.Light:SetIntensity(.9)
    --inst.Light:SetColour(255 / 255, 255 / 255, 180 / 255)
    --inst.Light:SetFalloff(0.3)
    --inst.Light:SetRadius(12)
	
	inst.persists = false
	inst:DoTaskInTime(90*FRAMES, function()
		inst.SoundEmitter:PlaySound("zg_laser/zg_laser/3")
	end)
    inst:ListenForEvent("animover", inst.Remove)
	
	return inst
end

local function fn_zg_icespike_fx_3()
	local inst = CreateEntity()
	
	inst.entity:AddTransform()
	--inst.Transform:SetScale(2, 2, 2)
	
	--inst.entity:AddSoundEmitter()
	--inst.entity:AddLight()
	inst.entity:AddAnimState()
	inst.entity:AddNetwork()	
	--
	inst.AnimState:SetBank("deerclops_icespike")
	inst.AnimState:SetBuild("deerclops_icespike")
	inst.AnimState:PlayAnimation("spike3")
	--inst.AnimState:SetOrientation(ANIM_ORIENTATION.OnGround)
    --inst.AnimState:SetLayer(LAYER_BACKGROUND)
    --inst.AnimState:SetSortOrder(3)	
	
	--
	inst:AddTag("FX")
	
	--inst.Light:Enable(true)
	--inst.Light:SetRadius(12)
	--inst.Light:SetFalloff(.3)
	--inst.Light:SetIntensity(0.9)
	--inst.Light:SetColour(255/255,255/255,180/255)
	
	inst.entity:SetPristine()

    if not TheWorld.ismastersim then
       return inst
    end
	
	--inst.Light:Enable(true)
    --inst.Light:SetIntensity(.9)
    --inst.Light:SetColour(255 / 255, 255 / 255, 180 / 255)
    --inst.Light:SetFalloff(0.3)
    --inst.Light:SetRadius(12)
	
	inst.persists = false
	--inst:DoTaskInTime(90*FRAMES, function()
	--	inst.SoundEmitter:PlaySound("zg_laser/zg_laser/3")
	--end)
    inst:ListenForEvent("animover", inst.Remove)
	
	return inst
end

return Prefab("deerclops", fn, assets, prefabs),
	Prefab("zg_laserfx", fn_laserfx),
	Prefab("zg_icespike_fx_3", fn_zg_icespike_fx_3)
	
	
	
	
	
	
	
	
	
