local assets =
{
    Asset("ANIM", "anim/balloon.zip"),
    Asset("ANIM", "anim/balloon_shapes.zip"),
}

local colours =
{
    { 198/255,  43/255,  43/255, 1 },
    {  79/255, 153/255,  68/255, 1 },
    {  35/255, 105/255, 235/255, 1 },
    { 233/255, 208/255,  69/255, 1 },
    { 109/255,  50/255, 163/255, 1 },
    { 222/255, 126/255,  39/255, 1 },
}

local easing = require("easing")

local function oncollide(inst, other)    
    if (inst:IsValid() and Vector3(inst.Physics:GetVelocity()):LengthSq() > .1) or
        (other ~= nil and other:IsValid() and Vector3(other.Physics:GetVelocity()):LengthSq() > .1) then
        inst.AnimState:PlayAnimation("hit")
        inst.AnimState:PushAnimation("idle", true)
    end
end

local function Explode(inst)
	RemovePhysicsColliders(inst)
	inst.AnimState:PlayAnimation("pop")
	inst.SoundEmitter:PlaySound("dontstarve/zg/zg_mo_evergreen/explode")
	local fx = SpawnPrefab("explode_small")
	fx.Transform:SetPosition(inst.Transform:GetWorldPosition())
	fx.Transform:SetScale(1.75,1.75,1.75)
	inst.DynamicShadow:Enable(false)
	inst:AddTag("NOCLICK")
	inst.persists = false
	inst.components.combat:DoAreaAttack(inst, 4)
	inst:Remove()
end

local function fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddDynamicShadow()
    inst.entity:AddNetwork()

    --MakeCharacterPhysics(inst, 10, .25)
	local phys = inst.entity:AddPhysics()
	phys:SetMass(10)
    phys:SetCapsule(.25, 1.5)
    phys:SetFriction(.3)
    phys:SetDamping(0)
	inst.Physics:SetRestitution(1)
    phys:SetCollisionGroup(COLLISION.CHARACTERS)
    phys:ClearCollisionMask()
	phys:CollidesWith(COLLISION.GROUND)
	phys:CollidesWith(COLLISION.OBSTACLES)
    phys:CollidesWith(COLLISION.SMALLOBSTACLES)

    inst.AnimState:SetBank("balloon")
    inst.AnimState:SetBuild("balloon")
    inst.AnimState:PlayAnimation("idle", true)
    inst.AnimState:SetRayTestOnBB(true)

    inst.DynamicShadow:SetSize(1, .5)
	
	inst:AddTag("zg_no_dmgid")

    --inst:AddTag("cattoyairborne")

    inst.entity:SetPristine()

    if not TheWorld.ismastersim then
        return inst
    end

    inst.Physics:SetCollisionCallback(oncollide)

    inst.AnimState:SetTime(math.random() * 2)

    inst.balloon_num = math.random(4)
    inst.AnimState:OverrideSymbol("swap_balloon", "balloon_shapes", "balloon_"..tostring(inst.balloon_num))
    inst.colour_idx = math.random(#colours)
    inst.AnimState:SetMultColour(unpack(colours[inst.colour_idx]))

    inst:AddComponent("inspectable")

    inst:AddComponent("combat")
    inst.components.combat:SetDefaultDamage( 60 * TUNING.ZG_DIFFICULTY )
	function inst.components.combat:DoAreaAttack(target, range, weapon, validfn, stimuli)
		local hitcount = 0
		local x, y, z = target.Transform:GetWorldPosition()
		local ents = TheSim:FindEntities(x, y, z, range, { "_combat" }, {"shadow", "shadowboss"})			----AOE伤害忽略队友
		for i, ent in ipairs(ents) do
			if ent ~= target and
				ent ~= self.inst and
				self:IsValidTarget(ent) and
				(validfn == nil or validfn(ent)) then
				self.inst:PushEvent("onareaattackother", { target = target, weapon = weapon, stimuli = stimuli })
				ent.components.combat:Zg_GetAttacked(self.inst, self:CalcDamage(ent, weapon, self.areahitdamagepercent), weapon, stimuli)
				hitcount = hitcount + 1
			end
		end
		return hitcount
	end
	
	inst:AddComponent("health")
    inst.components.health:SetMaxHealth(1)
    inst.components.health.nofadeout = true
	
	inst:ListenForEvent("death", Explode)
	
	inst:DoTaskInTime( 1 + math.random() * .5 , function(inst)
		Explode(inst)
	end)

    return inst
end

STRINGS.NAMES.ZG_IT_BALLOON = "Explode Balloon"
return Prefab("zg_it_balloon", fn, assets)
