local assets={
	Asset("SOUNDPACKAGE", "sound/zg_ghost.fev"),
	Asset("SOUND", "sound/zg_ghost.fsb"),
}

local book_defs =
{
    {
		name="abi",
		zg_name=zg_ischs("鬼魂兵器-阿比盖尔","Ghost Weapon-Abigail"),
		build="ghost_abigail_build",
		persists="no",
	},
}

local function MakeBook(def)
    
    local function fn()
        local inst = CreateEntity()

        inst.entity:AddTransform()
        inst.entity:AddAnimState()
        inst.entity:AddSoundEmitter()
        inst.entity:AddNetwork()

        MakeGhostPhysics(inst, 1, .5)

        inst.AnimState:SetBank(def.bank or "ghost")
        inst.AnimState:SetBuild(def.build)
        inst.AnimState:PlayAnimation(def.anim or "idle", true)

        inst.entity:SetPristine()
		
		inst:AddTag("healthinfo")
		inst:AddTag("hostile")
		
		inst:AddTag("scarytoprey")
		inst:AddTag("ghost")
		inst:AddTag("noauradamage")
		inst:AddTag("notraptrigger")
		
		inst:AddTag("zg_manshape")

        if not TheWorld.ismastersim then
            return inst
        end
		
		if def.persists=="no" then
			inst.persists=false
		end
		
		--引用
		inst.def=def

        --检测
		inst:AddComponent("inspectable")
		inst.components.inspectable:SetDescription(def.str or zg_ischs("鬼魂兵器","Ghost Weapon"))
		inst.name=def.zg_name or zg_ischs("鬼魂兵器","Ghost Weapon")
		
		--基础属性
		inst:AddComponent("talker")
		inst:AddComponent("health")
		inst.components.health:SetMaxHealth(def.health or 600)
		inst.components.health:SetAbsorptionAmount(def.absorb or 0)
		inst:AddComponent("locomotor")
		inst.components.locomotor:zg_setspeed(def.runspeed or 5,def.walkspeed or 4)
		inst.components.locomotor:SetTriggersCreep(false)
		inst:AddComponent("follower")
		
		--战斗属性
		inst:AddComponent("combat")
		inst.components.combat:SetDefaultDamage(def.damage or 20)
		inst.components.combat:SetRange(def.attackrange or 1)
		inst.components.combat:SetAttackPeriod(def.attackperiod or 1)
		inst.components.combat:SetRetargetFunction(def.retargetperiod or 1,def.retargetfn or function(inst)
			return FindEntity(inst,def.retargetdist or 30,
				function(guy)
					return inst.components.combat:CanTarget(guy) and (zg_isenemy(guy,inst,{"zg_manshape"}) or guy:HasTag("player"))
				end,
				nil,
				{"zg_manshape", "playerghost",}
			)
		end)
		inst.components.combat:SetKeepTargetFunction(def.keeptargetfn or function (inst, target)
			return inst.components.combat:CanTarget(target) and inst:IsNear(target,def.keeptargetdist or 40)
		end)
		inst:ListenForEvent("attacked",def.attackedfn or function(inst,data)
			local invader = nil
			invader = FindEntity(inst, inst.components.combat.attackrange+1, function(guy)
				return guy:HasTag("player") and not guy:HasTag("zg_manshape")
			end)
			if invader then
				inst.components.combat:SetTarget(invader)
			else
				inst.components.combat:SetTarget(data.attacker)
			end
		end)
		
		--掉落物
		inst:AddComponent("lootdropper")
		
		----	范围攻击
		inst:AddComponent("aura")
		inst.components.aura.radius = 2
		inst.components.aura.tickperiod = 1
		inst.components.aura.ignoreallies = true
		inst.components.aura.auratestfn = def.auratest or function(inst, target)
			return target and not target:HasTag("zg_manshape") and not target:HasTag("playerghost")
		end
		
		--SG
		--inst.zg_atksound=def.zg_atksound or "dontstarve/tentacle/tentacle_attack"
		inst.zg_hurtsound=def.zg_hurtsound or "dontstarve/wilson/hit"
		inst.zg_deathsound=def.zg_deathsound or "zg_ghost/zg_ghost/die"
		--inst.zg_ondeathfn=def.zg_ondeathfn and def.zg_ondeathfn(inst) or function() end
		inst:SetStateGraph(def.sg or "sg_ghost")
		
		--Behaviour
		inst.zg_chaseandattack_period=def.zg_chaseandattack_period or .1
		--inst.zg_runaway_period=def.zg_runaway_period or .1
		
		--AI
		inst.zg_ai_period=def.zg_ai_period or .1	--AI刷新时间
		inst.zg_chase_time=def.zg_chase_time or 30	--追杀时间
		--inst.zg_see_dist=def.zg_see_dist or 8	--逃跑检测hunter的距离
		
		--inst.zg_startrunaway_dist=def.zg_startrunaway_dist or 2		--开始躲避敌人预留的距离
		--inst.zg_stoprunaway_dist=def.zg_stoprunaway_dist or 3	--躲避敌人攻击距离 预留的安全距离
		
		--inst.target_exdist=def.target_exdist or 1	--玩家要是武器距离超过1,不再走位
		--inst.zg_findfood_dist=def.zg_findfood_dist or 10	--寻找食物的距离
		
		local brain=require "brains/br_ghost"
		inst:SetBrain(def.brain or brain)
		
        
        return inst
    end

    return Prefab("zg_mo1_"..def.name, fn, assets)
end

local books = {}
for i, v in ipairs(book_defs) do
    table.insert(books, MakeBook(v))
	STRINGS.NAMES[string.upper("zg_mo1_"..v.name)]=v.zg_name
end
book_defs = nil
return unpack(books)