local assets =
{
   
}

local prefabs =
{
    "houndstooth",
    "monstermeat",
    "redgem",
    "bluegem",
}

local brain = require "brains/zg_mo_hound"

SetSharedLootTable( 'hound',
{
    {'monstermeat', 1.000},
    {'houndstooth',  0.25},
})

SetSharedLootTable( 'hound_fire',
{
    {'monstermeat', 1.0},
    {'houndstooth', 1.0},
    {'houndfire',   1.0},
    {'houndfire',   1.0},
    {'houndfire',   1.0},
    {'redgem',      0.2},
})

SetSharedLootTable( 'hound_cold',
{
    {'monstermeat', 1.0},
    {'houndstooth', 1.0},
    {'houndstooth', 1.0},
    {'bluegem',     0.2},
})

local WAKE_TO_FOLLOW_DISTANCE = 8
local SLEEP_NEAR_HOME_DISTANCE = 10
local SHARE_TARGET_DIST = 30
local HOME_TELEPORT_DIST = 30

local NO_TAGS = {"FX", "NOCLICK","DECOR","INLIMBO"}

local function ShouldWakeUp(inst)
    return DefaultWakeTest(inst) or (inst.components.follower and inst.components.follower.leader and not inst.components.follower:IsNearLeader(WAKE_TO_FOLLOW_DISTANCE))
end

local function ShouldSleep(inst)
    return inst:HasTag("pet_hound")
    and not TheWorld.state.isday
    and not (inst.components.combat and inst.components.combat.target)
    and not (inst.components.burnable and inst.components.burnable:IsBurning())
    and (not inst.components.homeseeker or inst:IsNear(inst.components.homeseeker.home, SLEEP_NEAR_HOME_DISTANCE))
end

local function OnNewTarget(inst, data)
    if inst.components.sleeper:IsAsleep() then
        inst.components.sleeper:WakeUp()
    end
end

local function retargetfn(inst)
    local defenseTarget = inst
    local invader = nil
    invader = FindEntity(inst, 5 + 15 * 1, function(guy)
        return guy and not guy:HasTag("shadowboss") and not guy:HasTag("playerghost") and not guy:HasTag("hound") and not guy:HasTag("houndfriend") and
			(
				guy:HasTag("character") or guy:HasTag("animal") or guy:HasTag("monster")
			)
    end)
	if invader then
		if math.random() < .2 then
			inst.SoundEmitter:PlaySound("dontstarve/zg/cr_hound/call")
		end
	end

    return invader, true
end

local function KeepTarget(inst, target)
    return inst.components.combat:CanTarget(target) and (not inst:HasTag("pet_hound") or inst:IsNear(target, TUNING.HOUND_FOLLOWER_TARGET_KEEP))
end

local function OnAttacked(inst, data)
	

    inst.components.combat:SetTarget(data.attacker)
    inst.components.combat:ShareTarget(data.attacker, SHARE_TARGET_DIST, function(dude) return dude:HasTag("hound") or dude:HasTag("houndfriend") and not dude.components.health:IsDead() end, 5)
end

local function OnAttackOther(inst, data)
    inst.components.combat:ShareTarget(data.target, SHARE_TARGET_DIST, function(dude) return dude:HasTag("hound") or dude:HasTag("houndfriend") and not dude.components.health:IsDead() end, 5)
end

local function GetReturnPos(inst)
    local rad = 2
    local pos = inst:GetPosition()
    --print("GetReturnPos", inst, pos)
    local angle = math.random()*2*PI
    pos = pos + Point(rad*math.cos(angle), 0, -rad*math.sin(angle))
    --print("    ", pos)
    return pos:Get()
end

local function DoReturn(inst)
    --print("DoReturn", inst)
    if inst.components.homeseeker and inst.components.homeseeker:HasHome()  then
        if inst:HasTag("pet_hound") then
            if inst.components.homeseeker.home:IsAsleep() and not inst:IsNear(inst.components.homeseeker.home, HOME_TELEPORT_DIST) then
                local x, y, z = GetReturnPos(inst.components.homeseeker.home)
                inst.Physics:Teleport(x, y, z)
                --print("hound warped home", x, y, z)
            end
        elseif inst.components.homeseeker.home.components.childspawner then
            inst.components.homeseeker.home.components.childspawner:GoHome(inst)
        end
    end
end

local function OnEntitySleep(inst)
    --print("OnEntitySleep", inst)
    if not TheWorld.state.isday then
        DoReturn(inst)
    end
end

local function OnStopDay(inst)
    --print("OnStopDay", inst)
    if inst:IsAsleep() then
        DoReturn(inst)
    end
end

local function OnSave(inst, data)
    data.ispet = inst:HasTag("pet_hound")
    --print("OnSave", inst, data.ispet)
end

local function OnLoad(inst, data)
    --print("OnLoad", inst, data.ispet)
    if data and data.ispet then
        inst:AddTag("pet_hound")
        if inst.sg then
            inst.sg:GoToState("idle")
        end
    end
end

local function wilsononcollide(inst, other)
	if not other then
		return
	else
		if not other:IsValid() then
			return
		end
		----遇到障碍物就跳跃
		if inst.components.health and not inst.components.health:IsDead() and 
			not (other:HasTag("shadowboss") or other:HasTag("projectile") or other:HasTag("character") or other:HasTag("hound") or other:HasTag("frog")) then
			inst.sg:GoToState("mydarkjump")
		end
	end
end


local function fncommon(build)
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
	
    local phys = inst.entity:AddPhysics()
	phys:SetMass(10)
    phys:SetCapsule(.5, 1)
    phys:SetFriction(0)
    phys:SetDamping(5)
    phys:SetCollisionGroup(COLLISION.CHARACTERS)
    phys:ClearCollisionMask()
	phys:CollidesWith(COLLISION.GROUND)
	phys:CollidesWith(COLLISION.CHARACTERS)
	phys:CollidesWith(COLLISION.OBSTACLES)
    phys:CollidesWith(COLLISION.SMALLOBSTACLES)	
	
    inst.entity:AddSoundEmitter()
    inst.entity:AddDynamicShadow()
    inst.entity:AddNetwork()

    --MakeCharacterPhysics(inst, 10, .5)

    inst.DynamicShadow:SetSize(2.5, 1.5)
    inst.Transform:SetFourFaced()

    inst:AddTag("scarytoprey")
    inst:AddTag("monster")
    inst:AddTag("hostile")
    inst:AddTag("hound")
	--inst:AddTag("prey")

    inst.AnimState:SetBank("hound")
    inst.AnimState:SetBuild(build or "hound")
    inst.AnimState:PlayAnimation("idle")

    inst.entity:SetPristine()

    if not TheWorld.ismastersim then
        return inst
    end

    inst:AddComponent("locomotor") -- locomotor must be constructed before the stategraph
    inst.components.locomotor.runspeed = 10
    inst:SetStateGraph("SGmyhound")
	----	软实体的碰撞
	--inst.Physics:SetCollisionCallback(wilsononcollide)

    inst:SetBrain(brain)

    inst:AddComponent("follower")
	inst:AddComponent("talker")

    inst:AddComponent("eater")
    inst.components.eater:SetDiet({ FOODTYPE.MEAT }, { FOODTYPE.MEAT })
    inst.components.eater:SetCanEatHorrible()

    inst.components.eater.strongstomach = true -- can eat monster meat!

    inst:AddComponent("health")
    inst.components.health:SetMaxHealth(300)
	--inst.components.health:StartRegen(1,2)

    inst:AddComponent("sanityaura")
    inst.components.sanityaura.aura = -TUNING.SANITYAURA_MED

    inst:AddComponent("combat")
    inst.components.combat:SetDefaultDamage(30)
    inst.components.combat:SetAttackPeriod(3 - 1)
	inst.components.combat:SetRange(2.5)
    inst.components.combat:SetRetargetFunction(3, retargetfn)
    inst.components.combat:SetKeepTargetFunction(KeepTarget)
    --inst.components.combat:SetHurtSound("dontstarve/creatures/hound/hurt")
	
	

    inst:AddComponent("lootdropper")
    inst.components.lootdropper:SetChanceLootTable('hound')

    inst:AddComponent("inspectable")

    inst:AddComponent("sleeper")
    inst.components.sleeper:SetResistance(3)
    inst.components.sleeper.testperiod = GetRandomWithVariance(6, 2)
    inst.components.sleeper:SetSleepTest(ShouldSleep)
    inst.components.sleeper:SetWakeTest(ShouldWakeUp)
    inst:ListenForEvent("newcombattarget", OnNewTarget)

	--[[
    MakeHauntableChangePrefab(inst, {"firehound", "icehound"})
    inst:ListenForEvent("spawnedfromhaunt", function(inst)
        if inst.components.hauntable then
            inst.components.hauntable:Panic()
        end
    end)]]

    inst:WatchWorldState("stopday", OnStopDay)
    inst.OnEntitySleep = OnEntitySleep

    inst.OnSave = OnSave
    inst.OnLoad = OnLoad

    inst:ListenForEvent("attacked", OnAttacked)
    inst:ListenForEvent("onattackother", OnAttackOther)

    return inst
end

local function fndefault()
    local inst = fncommon()

    if not TheWorld.ismastersim then
        return inst
    end

    MakeMediumFreezableCharacter(inst, "hound_body")
    MakeMediumBurnableCharacter(inst, "hound_body")
	--[[
	----变异变回去
	inst:AddComponent("zg_creature_variation")
	inst.components.zg_creature_variation:SetVariationTime(30 * 3)
	inst.components.zg_creature_variation:StartVariation()
	inst.components.zg_creature_variation.onvariationreplacement = "hound"
	inst.components.zg_creature_variation.chance = .15
	]]
    return inst
end

local function PlayFireExplosionSound(inst)
    inst.SoundEmitter:PlaySound("dontstarve/creatures/hound/firehound_explo", "explosion")
end

local function fnfire()
    local inst = fncommon("hound_red")

    if not TheWorld.ismastersim then
        return inst
    end

    MakeMediumFreezableCharacter(inst, "hound_body")
    inst.components.freezable:SetResistance(4) --because fire

    inst.components.combat:SetDefaultDamage(45)
    inst.components.combat:SetAttackPeriod(3 - 1)
    inst.components.locomotor.runspeed = TUNING.FIREHOUND_SPEED
    inst.components.health:SetMaxHealth(200)
    inst.components.lootdropper:SetChanceLootTable('hound_fire')

    inst:ListenForEvent("death", PlayFireExplosionSound)
	
	----变异变回去
	--[[
	inst:AddComponent("zg_creature_variation")
	inst.components.zg_creature_variation:SetVariationTime(30 * 3)
	inst.components.zg_creature_variation:StartVariation()
	inst.components.zg_creature_variation.onvariationreplacement = "firehound"
	inst.components.zg_creature_variation.chance = .15
]]
    return inst
end

local function DoIceExplosion(inst)
     if not inst.components.freezable then
            MakeMediumFreezableCharacter(inst, "hound_body")
        end
        inst.components.freezable:SpawnShatterFX()
        inst:RemoveComponent("freezable")
        local x,y,z = inst.Transform:GetWorldPosition()
        local ents = TheSim:FindEntities(x, y, z, 4, {"freezable"}, NO_TAGS) 
        for i,v in pairs(ents) do
            if v.components.freezable and not v:HasTag("hound") then
                v.components.freezable:AddColdness(2)
            end
        end

    inst.SoundEmitter:PlaySound("dontstarve/creatures/hound/icehound_explo", "explosion")
end

local function fncold()
    local inst = fncommon("hound_ice")

    if not TheWorld.ismastersim then
        return inst
    end

    MakeMediumBurnableCharacter(inst, "hound_body")

    inst.components.combat:SetDefaultDamage(45)
    inst.components.combat:SetAttackPeriod(3 - 1)
    inst.components.locomotor.runspeed = TUNING.ICEHOUND_SPEED
    inst.components.health:SetMaxHealth(200)
    inst.components.lootdropper:SetChanceLootTable('hound_cold')

    inst:ListenForEvent("death", DoIceExplosion)
	--[[
	----变异变回去
	inst:AddComponent("zg_creature_variation")
	inst.components.zg_creature_variation:SetVariationTime(30 * 3)
	inst.components.zg_creature_variation:StartVariation()
	inst.components.zg_creature_variation.onvariationreplacement = "icehound"
	inst.components.zg_creature_variation.chance = .15
]]
    return inst
end

local function fnfiredrop()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddNetwork()

    MakeInventoryPhysics(inst)

    inst.entity:SetPristine()

    if not TheWorld.ismastersim then
        return inst
    end

    MakeLargeBurnable(inst, 6 + math.random() * 6)
    MakeLargePropagator(inst)

    --Remove the default handlers that toggle persists flag
    inst.components.burnable:SetOnIgniteFn(nil)
    inst.components.burnable:SetOnExtinguishFn(inst.Remove)
    inst.components.burnable:Ignite()

    return inst
end

STRINGS.NAMES.ZG_MO_HOUND = "精英猎犬"
STRINGS.NAMES.ZG_MO_FIREHOUND = "精英火狗"
STRINGS.NAMES.ZG_MO_ICEHOUND = "精英冰狗"
return Prefab("zg_mo_hound", fndefault, assets, prefabs),
        Prefab("zg_mo_firehound", fnfire, assets, prefabs),
        Prefab("zg_mo_icehound", fncold, assets, prefabs)
        --Prefab("monsters/houndfire", fnfiredrop, assets, prefabs)
