require("stategraphs/commonstates")

local function DoFoleySounds(inst)

	for k,v in pairs(inst.components.inventory.equipslots) do
		if v.components.inventoryitem and v.components.inventoryitem.foleysound then
			inst.SoundEmitter:PlaySound(v.components.inventoryitem.foleysound)
		end
	end

end

----子弹用完后装备近战武器
local function GetMeeleWeaopn(inst)
	if inst.components.inventory ~= nil then
		local meleeweapon = CreateEntity()
		meleeweapon.entity:AddTransform()
		meleeweapon:AddComponent("weapon")
		meleeweapon.components.weapon:SetDamage(30 * TUNING.ZG_DIFFICULTY)
		meleeweapon:AddComponent("inventoryitem")
		meleeweapon.persists = false
		meleeweapon.components.inventoryitem:SetOnDroppedFn(inst.Remove)
		meleeweapon:AddComponent("equippable")
		meleeweapon.components.equippable:SetOnEquip(function()
			inst.AnimState:OverrideSymbol("swap_object", "zg_swap_axe", "swap_axe")
			inst.AnimState:Show("ARM_carry") 
			inst.AnimState:Hide("ARM_normal")
		end)
		meleeweapon:AddTag("meleeweapon")
		inst.components.inventory:Equip(meleeweapon)
	end
end

local actionhandlers = 
{
	ActionHandler(ACTIONS.SHAVE, "axe_dance"),
}

   
local events=
{

	--CommonHandlers.OnSleep(),
	EventHandler("gotosleep", function (inst)
		if inst.components.health ~= nil and not inst.components.health:IsDead() and not inst.sg:HasStateTag("sleep") then
			inst.sg:GoToState("sleep")
		end
	end),
	
    CommonHandlers.OnFreeze(),
	--CommonHandlers.OnAttacked(true),

	CommonHandlers.OnLocomote(true, false),

    EventHandler("attacked", function(inst, data)
		if inst.components.health and not inst.components.health:IsDead() and not inst.sg:HasStateTag("attack") then
            if inst.sg:HasStateTag("sleeping") then
				inst.sg:GoToState("wakeup")
			else
				inst.sg:GoToState("hit")
			end
        end
	end),

    EventHandler("doattack", function(inst, data)
        if not inst.components.health:IsDead() then
			local equip = inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS)
			if equip and equip.name == "shortgun" then
				inst.sg:GoToState("attack_shortgun", data.target)
			elseif equip and equip.name == "prifle" then
				inst.sg:GoToState("attack_prifle", data.target)
			elseif equip and equip.name == "boomerangaxe" then
				inst.sg:GoToState("attack_boomerangaxe", data.target)
			else
				inst.sg:GoToState("attack", data.target)
			end
        end
    end),
    
    EventHandler("death", function(inst)
        inst.sg:GoToState("death")
    end),
}

local states= 
{

	State{
        name = "sleep",
        tags = { "busy", "sleep"},

        onenter = function(inst)
            inst.components.locomotor:Stop()
            
            inst.AnimState:PlayAnimation("dozy")
            
            inst.sg:SetTimeout(TUNING.KNOCKOUT_SLEEP_TIME)
        end,

        ontimeout = function(inst)
            inst.sg:GoToState("wakeup")
        end,

        events =
        {
            EventHandler("firedamage", function(inst)
                if inst.sg:HasStateTag("sleeping") then
                    inst.sg:GoToState("wakeup")
                end
            end),
            EventHandler("animover", function(inst)
                if inst.AnimState:AnimDone() then
					inst.AnimState:PlayAnimation("sleep_loop", true)
					inst.sg:AddStateTag("sleeping")
                end
            end),
        },
    },

	State{
        name = "wakeup",
        tags = { "busy", },

        onenter = function(inst)
            inst.AnimState:PlayAnimation("wakeup")
        end,

        events =
        {
            EventHandler("animover", function(inst)
                if inst.AnimState:AnimDone() then
                    inst.sg:GoToState("idle")
                end
            end),
        },
    },

------------------------------------------------------------------------------------------------

	State{
        name = "hit",
        tags = { "busy", },

        onenter = function(inst)
            inst.components.locomotor:Stop()
            inst.AnimState:PlayAnimation("hit")
            local stun_frames = 6
            inst.sg:SetTimeout(stun_frames * FRAMES)
        end,

        ontimeout = function(inst)
            inst.sg:GoToState("idle")
        end,
    },

	----跳舞
	State{
        name = "axe_dance",
        tags = {"busy", "axe_dance"},
        
        onenter = function(inst)
            inst.components.locomotor:Stop()
            inst.AnimState:PlayAnimation("emoteXL_loop_dance0")
        end,
		
		events=
        {
            EventHandler("animover", function(inst)
                inst.sg:GoToState("axe_dance")
            end ),
        },
		
    },

    State{
        name = "death",
        tags = {"busy"},
        
        onenter = function(inst)
            inst.components.locomotor:Stop()
			inst.SoundEmitter:PlaySound("dontstarve/zg/wilson/die")
            inst.AnimState:Hide("swap_arm_carry")
            inst.AnimState:PlayAnimation("death")
            inst.components.lootdropper:DropLoot(Vector3(inst.Transform:GetWorldPosition()))  
        end,
    },

    State{
        name = "idle",
        tags = {"idle", "canrotate"},
        onenter = function(inst, pushanim)     
            inst.components.locomotor:Stop()
			if math.random() < .1 then
				inst.SoundEmitter:PlaySound("dontstarve/zg/wilson/call")
			end
            local anims = {"idle_loop"}          
            local anim = "idle_loop"                      
            if pushanim then
                for k,v in pairs (anims) do
					inst.AnimState:PushAnimation(v, k == #anims)
				end
            else
                inst.AnimState:PlayAnimation(anims[1], #anims == 1)
                for k,v in pairs (anims) do
					if k > 1 then
						inst.AnimState:PushAnimation(v, k == #anims)
					end
				end
            end  
            inst.sg:SetTimeout(1)
        end,
    },
	
------------------------------------------------------------------------------------------------------------------------------------------------------
    State{
        name = "attack",
        tags = {"attack", "notalking", "abouttoattack", "busy"},
        
        onenter = function(inst, target)
            inst.sg.statemem.target = target
            inst.components.combat:StartAttack()
			inst.components.locomotor:Stop()
			local animrandom = math.random()
			if animrandom < .33 then
				inst.AnimState:PlayAnimation("chop_loop")
			elseif animrandom < .66 then
				inst.AnimState:PlayAnimation("atk")
			else
				inst.AnimState:PlayAnimation("pickaxe_loop")
			end
        end,
        
        timeline=
        {	
			TimeEvent(4*FRAMES, function(inst) inst.SoundEmitter:PlaySound("dontstarve/tentacle/tentacle_attack") end),
            TimeEvent(8*FRAMES, function(inst) 
				inst.components.combat:DoAttack(inst.sg.statemem.target) 
				inst.sg:RemoveStateTag("abouttoattack") 
			end),  
			TimeEvent(10*FRAMES, function(inst) 
				inst.sg:RemoveStateTag("busy")
				inst.sg:RemoveStateTag("attack")
			end),
        },
        
        events=
        {
            EventHandler("animover", function(inst)
                inst.sg:GoToState("idle")
            end ),
        },
    },  
	
	State{
        name = "attack_boomerangaxe",
        tags = {"attack", "notalking", "abouttoattack", "busy"},
        
        onenter = function(inst, target)
            inst.sg.statemem.target = target
            inst.components.combat:StartAttack()
			inst.components.locomotor:Stop()
			inst.AnimState:PlayAnimation("throw")
        end,
        
        timeline=
        {	
			TimeEvent(7 * FRAMES, function(inst)
                --inst:PerformBufferedAction()
				inst.components.combat:DoAttack(inst.sg.statemem.target) 
                inst.sg:RemoveStateTag("abouttoattack")
				inst.sg:RemoveStateTag("attack")
				inst.sg:RemoveStateTag("busy")
            end),
        },
        
        events=
        {
            EventHandler("animover", function(inst)
                inst.sg:GoToState("idle")
            end ),
        },
    },  

	State{
        name = "attack_shortgun",
        tags = {"attack", "notalking", "abouttoattack", "busy"},
        
        onenter = function(inst, target)
            inst.sg.statemem.target = target
            inst.components.combat:StartAttack()
			inst.components.locomotor:Stop()
			
			inst.AnimState:PlayAnimation("dart")
			
        end,
        
        timeline=
        {	
			TimeEvent(6*FRAMES, function(inst) 
				inst.SoundEmitter:PlaySound("dontstarve/zg/ch_sanji/atk") 
				----在前方2处产生火花
				local pt = inst:GetPosition()
				local heading_angle = -(inst.Transform:GetRotation())
				local r = 2
				local dir = Vector3(math.cos(heading_angle*DEGREES) * r, 1.675, math.sin(heading_angle*DEGREES) * r)
				local fx = SpawnPrefab("zg_fx_gunfire")
				fx.Transform:SetPosition( (pt + dir):Get() )
				fx.Transform:SetScale(.25,.25,.25)
			end),
            TimeEvent(10*FRAMES, function(inst) 
				inst.components.combat:DoAttack(inst.sg.statemem.target) 
				inst.sg:RemoveStateTag("abouttoattack") 
			end),  
			TimeEvent(12*FRAMES, function(inst) 
				inst.sg:RemoveStateTag("busy")
				inst.sg:RemoveStateTag("attack")
				
				inst.zg_bullet = inst.zg_bullet - 1
				if inst.zg_bullet <= 0 then
					GetMeeleWeaopn(inst)
				end
			end),
        },
        
        events=
        {
            EventHandler("animover", function(inst)
                inst.sg:GoToState("idle")
            end ),
        },
    },  
	
	State{
        name = "attack_prifle",
        tags = {"attack", "notalking", "abouttoattack", "busy"},
        
        onenter = function(inst, target)
            inst.sg.statemem.target = target
            inst.components.combat:StartAttack()
			inst.components.locomotor:Stop()
			
			inst.components.combat:SetAttackPeriod(0)
			
			inst.AnimState:PlayAnimation("dart")
			inst.sg:SetTimeout(4*FRAMES)
			
        end,
        
        timeline=
        {	
			TimeEvent(1*FRAMES, function(inst) 
				--inst.SoundEmitter:PlaySound("dontstarve/common/destroy_smoke") 
				--inst.SoundEmitter:PlaySound("dontstarve/zg/ch0_sweeper/mini") 
				inst.SoundEmitter:PlaySound("zg_we_mini/zg_we_mini/atk1")
				----在前方2处产生火花
				local pt = inst:GetPosition()
				local heading_angle = -(inst.Transform:GetRotation())
				local r = 2
				local dir = Vector3(math.cos(heading_angle*DEGREES) * r, 1.675, math.sin(heading_angle*DEGREES) * r)
				local fx = SpawnPrefab("zg_fx_gunfire")
				fx.Transform:SetPosition( (pt + dir):Get() )
				fx.Transform:SetScale(.25,.25,.25)
			end),
            TimeEvent(2*FRAMES, function(inst) 
				inst.components.combat:DoAttack(inst.sg.statemem.target) 
				inst.sg:RemoveStateTag("abouttoattack") 
			end),  
			TimeEvent(3*FRAMES, function(inst) 
				inst.sg:RemoveStateTag("busy")
				inst.sg:RemoveStateTag("attack")
				inst.zg_bullet = inst.zg_bullet - 1
				if inst.zg_bullet <= 0 then
					GetMeeleWeaopn(inst)
				end
			end),
        },
        
        ontimeout = function(inst)
			if math.random() < .2 then
				inst.components.combat:SetTarget(nil)
			end
			inst.sg:GoToState("idle")
		end,
		
		onexit = function(inst)
			--inst.components.combat:SetAttackPeriod(1)
		end,
    },  
   
    State{
        name = "run_start",
        tags = {"moving", "running", "canrotate"},
        
        onenter = function(inst)
			inst.components.locomotor:RunForward()
            inst.AnimState:PlayAnimation("run_pre")
            inst.sg.mem.foosteps = 0
        end,

        onupdate = function(inst)
            inst.components.locomotor:RunForward()
        end,

        events=
        {   
            EventHandler("animover", function(inst) inst.sg:GoToState("run") end ),        
        },
        
        timeline=
        {
        
            TimeEvent(4*FRAMES, function(inst)
                PlayFootstep(inst)
                DoFoleySounds(inst)
            end),
        },        
        
    },

    State{
        
        name = "run",
        tags = {"moving", "running", "canrotate"},
        
        onenter = function(inst) 
            inst.components.locomotor:RunForward()
            inst.AnimState:PlayAnimation("run_loop")
            
        end,
        
        onupdate = function(inst)
            inst.components.locomotor:RunForward()
        end,

        timeline=
        {
            TimeEvent(7*FRAMES, function(inst)
				inst.sg.mem.foosteps = inst.sg.mem.foosteps + 1
                PlayFootstep(inst, inst.sg.mem.foosteps < 5 and 1 or .6)
                DoFoleySounds(inst)
            end),
            TimeEvent(15*FRAMES, function(inst)
				inst.sg.mem.foosteps = inst.sg.mem.foosteps + 1
                PlayFootstep(inst, inst.sg.mem.foosteps < 5 and 1 or .6)
                DoFoleySounds(inst)
            end),
        },
        
        events=
        {   
            EventHandler("animover", function(inst) inst.sg:GoToState("run") end ),        
        },
        
        
    },
    
    State{
        name = "run_stop",
        tags = {"canrotate", "idle"},
        
        onenter = function(inst) 
            inst.components.locomotor:Stop()
            inst.AnimState:PlayAnimation("run_pst")
        end,
        
        events=
        {   
            EventHandler("animover", function(inst) inst.sg:GoToState("idle") end ),        
        },       
    },  
		
	State {
		name = "frozen",
		tags = {"busy"},
		
        onenter = function(inst)
            inst.AnimState:PlayAnimation("frozen")
            inst.Physics:Stop()
        end,
    },
}
CommonStates.AddFrozenStates(states)
    
return StateGraph("zg_ch0_sweeper", states, events, "idle", actionhandlers)

