require("stategraphs/commonstates")

local function DoFoleySounds(inst)
	for k,v in pairs(inst.components.inventory.equipslots) do
		if v.components.inventoryitem and v.components.inventoryitem.foleysound then
			inst.SoundEmitter:PlaySound(v.components.inventoryitem.foleysound)
		end
	end

end

local actionhandlers = 
{
	ActionHandler(ACTIONS.EAT, "quickeat"),
}

   
local events=
{
	CommonHandlers.OnLocomote(true, false),
	CommonHandlers.OnFreeze(),
	
	--武器坏了
	EventHandler("toolbroke", function(inst, data)
		local tool=data.tool
		--对于改变状态的道具,不做这个处理
		if not tool:HasTag("zg_makehole") then
			inst.sg:GoToState("toolbroke", tool)
		end
	end),

    EventHandler("attacked", function(inst, data)
		if zg_isalive(inst) then
			--被打声音
			inst.SoundEmitter:PlaySound(inst.zg_hurtsound or "")
			--机器人则冒火花
			if inst.zg_isrobot then
				zg_spawn(inst,"sparks")
			--骨头人掉渣
			elseif inst.zg_isskel then
				zg_spawn(inst,"brokentool")
			--肉体
			else
				zg_spawn(inst,"zg_fx_blood")
				--local fx=zg_spawn(inst,"mining_fx")
				--if not fx.AnimState then fx.entity:AddAnimState() end
				--fx.AnimState:SetMultColour(.5,0,0,1)
			end
			--
			if data.attacker and data.attacker:IsNear(inst,3) and not inst.sg:HasStateTag("attack") then
				inst.sg:GoToState("hit")
			end
		end
	end),

    EventHandler("doattack", function(inst,data)
        if zg_isalive(inst) then
			local equip = inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS)
			if not equip then inst:EquipRandomWeapon(inst) end
			inst.sg:GoToState("attack", data.target)
        end
    end),
    
    EventHandler("death", function(inst)
        inst.sg:GoToState("death")
    end),
}

local states= 
{

	State{
        name = "jump",
        tags = {"attack","busy"},
        
        onenter = function(inst)
			
			inst.Physics:ClearCollisionMask()
			inst.Physics:CollidesWith(COLLISION.GROUND)
			inst.Physics:CollidesWith(COLLISION.WORLD)
	
			inst.AnimState:PlayAnimation("jumpout")
			inst.Physics:SetMotorVel(40, 0, 0)
			
			inst.sg:SetTimeout(17*FRAMES)
			
        end,
        
		onupdate = function(inst)         
            inst.components.locomotor:RunForward()			----	直接冲过去
        end,
		
		onexit = function(inst)
			inst.Physics:Stop()
			inst.Physics:ClearCollisionMask()
			inst.Physics:CollidesWith(COLLISION.GROUND)
			inst.Physics:CollidesWith(COLLISION.OBSTACLES)
			inst.Physics:CollidesWith(COLLISION.SMALLOBSTACLES)
			inst.Physics:CollidesWith(COLLISION.CHARACTERS)
			
			local x,y,z = inst.Transform:GetWorldPosition()
			inst.Transform:SetPosition(x,0,z)
			
			
		end,
		
        timeline=
        {	
			TimeEvent(15 * FRAMES, function(inst)
				inst.SoundEmitter:PlaySound("dontstarve/movement/bodyfall_dirt")
				
			end),
        },
		
		ontimeout=function(inst)
			inst.sg:GoToState( inst.jump_outstate or "attack",inst.components.combat.target)
		end,
       
    },    
	
	--死亡
    State{
        name = "death",
        tags = {"busy"},
        
        onenter = function(inst)
            inst.components.locomotor:Stop()
			--确保死的时候在地上
			local x,y,z=inst.Transform:GetWorldPosition()
			inst.Transform:SetPosition(x,0,z)
			--死亡声音
			inst.SoundEmitter:PlaySound(inst.zg_deathsound or "")
            inst.AnimState:Hide("swap_arm_carry")
            inst.AnimState:PlayAnimation("death")
            inst.components.lootdropper:DropLoot(Vector3(inst.Transform:GetWorldPosition())) 
			--死亡事件
			inst:zg_ondeathfn(inst)
        end,
    },

	--静止
    State{
        name = "idle",
        tags = {"idle", "canrotate"},
        onenter = function(inst, pushanim)     
            inst.components.locomotor:Stop()
			
			if inst.hasshield then
				inst.AnimState:PlayAnimation("zg_idle_loop",true)
			else
				inst.AnimState:PlayAnimation("idle_loop",true)
			end
			--[[
            local anims = {"idle_loop"}          
            local anim = "idle_loop"                      
            if pushanim then
                for k,v in pairs (anims) do
					inst.AnimState:PushAnimation(v, k == #anims)
				end
            else
                inst.AnimState:PlayAnimation(anims[1], #anims == 1)
                for k,v in pairs (anims) do
					if k > 1 then
						inst.AnimState:PushAnimation(v, k == #anims)
					end
				end
            end  ]]
            inst.sg:SetTimeout(math.random()*4+2)
        end,
    },

	--吃
    State{
        name = "quickeat",
        tags ={"busy"},
        onenter = function(inst)
            inst.components.locomotor:Stop()
            inst.AnimState:PlayAnimation("quick_eat")
        end,

        timeline=
        {
            TimeEvent(12*FRAMES, function(inst) 
                inst:PerformBufferedAction() 
                inst.sg:RemoveStateTag("busy")
            end),
        },        
        
        events=
        {
            EventHandler("animover", function(inst) inst.sg:GoToState("idle") end),
        },
        
        onexit= function(inst)
            inst.SoundEmitter:KillSound("eating")    
        end,
    },    
	
	--攻击
    State{
        name = "attack",
        tags = {"attack","busy"},
		
		--onexit=function(inst)
			
		--end,
        
        onenter = function(inst,tar)
		
			local equip = inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS)
		
			--特殊情况
			if equip and equip:HasTag("zg_summon") then
				inst.sg:GoToState("summon",tar)
				return
			elseif equip and equip:HasTag("zg_summon2") then
				inst.sg:GoToState("summon2",tar)
				return
			elseif equip and equip:HasTag("zg_summon3") then
				inst.sg:GoToState("summon3",tar)
				return
			elseif equip and equip:HasTag("zg_readbook") then
				inst.sg:GoToState("readbook",tar)
				return
			elseif equip and equip:HasTag("zg_makehole") then
				inst.sg:GoToState("makehole",tar)
				return
			elseif equip and equip:HasTag("zg_bow") then
				inst.sg:GoToState("bow",tar)
				return
			elseif equip and equip:HasTag("zg_gun") then
				inst.sg:GoToState("gun",tar)
				return
				
			--突击
			elseif equip and equip:HasTag("z_s_ass") then
				inst.sg:GoToState("z_s_ass",tar)
				return
			end
			
            inst.sg.statemem.target = tar
            inst.components.combat:StartAttack()
			inst.components.locomotor:Stop()
			
			--普通格斗武器
			if equip ~= nil and equip:HasTag("whip") then
                inst.AnimState:PlayAnimation("whip_pre")
                inst.AnimState:PushAnimation("whip", false)
                inst.SoundEmitter:PlaySound("dontstarve/common/whip_pre", nil, nil, true)
				inst.sg.statemem.iswhip=true
			elseif equip ~= nil and equip:HasTag("zg_spear") then
				inst.AnimState:PlayAnimation("spearjab")
				inst.sg.statemem.isspear=true
				--inst.SoundEmitter:PlaySound("zg_melee/spear/atk3", nil, nil, true)
				
			elseif equip ~= nil and equip.components.weapon ~= nil and not equip:HasTag("punch") then
			
                if inst.hasshield then
					inst.AnimState:PlayAnimation("zg_atk_pre")
					inst.AnimState:PushAnimation("zg_atk", false)
				else
					inst.AnimState:PlayAnimation("atk_pre")
					inst.AnimState:PushAnimation("atk", false)
				end
				
                inst.SoundEmitter:PlaySound(
				
					(equip:HasTag("zg_heavy_axe") and "zg_melee/spear/atk3") or
					
                    (equip:HasTag("icestaff") and "dontstarve/wilson/attack_icestaff") or
                    (equip:HasTag("shadow") and "dontstarve/wilson/attack_nightsword") or
                    (equip:HasTag("firestaff") and "dontstarve/wilson/attack_firestaff") or
                    "dontstarve/wilson/attack_weapon",
                    nil, nil, true
                )
			elseif equip ~= nil and (equip:HasTag("light") or equip:HasTag("nopunch")) then
                if inst.hasshield then
					inst.AnimState:PlayAnimation("zg_atk_pre")
					inst.AnimState:PushAnimation("zg_atk", false)
				else
					inst.AnimState:PlayAnimation("atk_pre")
					inst.AnimState:PushAnimation("atk", false)
				end
				
                inst.SoundEmitter:PlaySound("dontstarve/wilson/attack_weapon", nil, nil, true)
			else
                inst.AnimState:PlayAnimation("punch")
                inst.SoundEmitter:PlaySound("dontstarve/wilson/attack_whoosh", nil, nil, true)
			end
			inst.sg:SetTimeout(9*FRAMES)
        end,
        
        timeline=
        {	
			TimeEvent(2*FRAMES, function(inst)
				if inst.sg.statemem.isspear then
					--声音
					local weapon = inst.components.combat and inst.components.combat:GetWeapon()
					if weapon then
						inst.SoundEmitter:PlaySound( weapon:HasTag("zg_heavy") and "zg_melee/spear/atk3" or "dontstarve/tentacle/tentacle_attack") 
					end
				end
			end),
			TimeEvent(6 * FRAMES, function(inst)
                if not (inst.sg.statemem.isspear or inst.sg.statemem.iswhip) then
                    inst.components.combat:DoAttack(inst.sg.statemem.target)
                end
            end),
            TimeEvent(7 * FRAMES, function(inst)	--离timeout至少要2帧
                if inst.sg.statemem.iswhip or inst.sg.statemem.isspear then
                    inst.components.combat:DoAttack(inst.sg.statemem.target)
                end
            end), 
        },
		
		ontimeout = function(inst)
			if inst.num_attack_to_swap then
				inst.num_attack=(inst.num_attack or 0) + 1
				if inst.num_attack>=inst.num_attack_to_swap then
					inst.num_attack=inst.num_attack-inst.num_attack_to_swap
					inst:EquipRandomWeapon(inst)
				end
			end
            inst.sg:GoToState("idle", true)
        end,
       
    },  
------------------------------------------------------------------------------------------------------
	--突击
	State{
		name="z_s_ass",
		tags = {"attack","jumping","canrotate"},	--jumping不能少,如果没有该标签会被behavior结点里被stop
		onenter = function(inst,target)
			inst.components.locomotor:Stop()
			inst.sg.statemem.target = target
			inst.components.combat:StartAttack()
			inst.AnimState:PlayAnimation("spearjab")
			inst.sg:SetTimeout(15*FRAMES)
		end,
		timeline =
		{		
			TimeEvent(2*FRAMES, function(inst) --突击
				--突击音效
				local weapon = inst.components.combat and inst.components.combat:GetWeapon()
				if weapon then inst.SoundEmitter:PlaySound("zg_melee/skill/assult") end
				
				inst.Physics:SetMotorVel(40,0,0)
				zg_spawn(inst,"maxwell_smoke")
			end),
	
			--
			TimeEvent(5*FRAMES, function(inst) 
				if inst.sg.statemem.target then
					if distsq(inst.sg.statemem.target:GetPosition(),inst:GetPosition()) <= (inst.components.combat.hitrange)*(inst.components.combat.hitrange) then
						inst.Physics:Stop()	--停止
						inst.components.locomotor:Stop()
					else
						inst.Physics:SetMotorVel(40,0,0)
					end
				end
				zg_spawn(inst,"maxwell_smoke")
			end),
			
			TimeEvent(8*FRAMES, function(inst) 
				inst.SoundEmitter:PlaySound("zg_melee/spear/atk3")	--攻击音效
				if inst.sg.statemem.target then
					if distsq(inst.sg.statemem.target:GetPosition(),inst:GetPosition()) <= (inst.components.combat.hitrange)*(inst.components.combat.hitrange) then
						inst.Physics:Stop()	--停止
						inst.components.locomotor:Stop()
					else
						inst.Physics:SetMotorVel(40,0,0)
					end
				end
				zg_spawn(inst,"maxwell_smoke")
			end),
			
			TimeEvent(11*FRAMES, function(inst) 	--实施打击
				inst.components.combat:DoAttack(inst.sg.statemem.target) 
			end),  
			TimeEvent(12*FRAMES, function(inst) 
				inst.Physics:Stop()	--停止
				inst.components.locomotor:Stop()
				
				local weapon = inst.components.combat and inst.components.combat:GetWeapon()
				if weapon and weapon.components.finiteuses then
					weapon.components.finiteuses:Use(1)
				end
			end),			
		},
		
		ontimeout=function(inst)
			inst.sg:GoToState("idle")
		end,
	},
	
------------------------------------------------------------------------------------------------------	
	--枪类
	State{
		name="gun",
		tags = { "attack" },
		onenter = function(inst,target)
			inst.sg.statemem.target = target
			inst.components.combat:StartAttack()
			inst.components.locomotor:Stop()
			inst.AnimState:PlayAnimation("fishing_idle")
			--inst.sg:SetTimeout(9*FRAMES)
		end,
		timeline =
		{		
			TimeEvent(2*FRAMES, function(inst) 
				--枪的声音
				local weapon = inst.components.combat and inst.components.combat:GetWeapon()
				--火枪
				if weapon and weapon:HasTag("zg_gun_fire") then
					inst.SoundEmitter:PlaySound("zg_melee/gun/atk_fire")
				end
			end),
			TimeEvent(3*FRAMES, function(inst) 
				inst.components.combat:DoAttack(inst.sg.statemem.target) 
			end),  
			TimeEvent(4*FRAMES, function(inst) 
				--消耗耐久在开枪后 而不是 饥荒传统的击中算法
				local weapon = inst.components.combat and inst.components.combat:GetWeapon()
				if weapon and weapon.components.finiteuses then
					weapon.components.finiteuses:Use(1)
				end
			end),			
		},
		events =
		{
			EventHandler("animover", function(inst)
				inst.sg:GoToState("idle")
			end ),
		},
	},

	--弓箭
	State{
		name = "bow",
		tags = {"attack"},

		onenter = function(inst, target)
			inst.sg.statemem.target = target
			inst.components.combat:StartAttack()
			inst.components.locomotor:Stop()
			inst.AnimState:PlayAnimation("bow_attack")
			--inst.sg:SetTimeout(9*FRAMES)
		end,

		timeline =
		{		
			TimeEvent(4*FRAMES, function(inst) 
				inst.SoundEmitter:PlaySound("zg_bow/zg_bow/atk")
			end),
            TimeEvent(7*FRAMES, function(inst) 
				inst.components.combat:DoAttack(inst.sg.statemem.target) 
			end),  		
		},

		events =
		{
			EventHandler("animover", function(inst)
                inst.sg:GoToState("idle")
            end ),
		},
	},
	
	--打洞
	State{
        name = "makehole",
        tags = { "attack", "busy", "canrotate"},

        onenter = function(inst,tar)
			inst.sg.statemem.target = tar
            inst.components.combat:StartAttack()
			inst.components.locomotor:Stop()
			
            inst.AnimState:PlayAnimation("pickaxe_loop")
            inst.AnimState:PushAnimation("pickaxe_loop", false)
			--inst.AnimState:PushAnimation("pickaxe_loop", false)
            inst.components.locomotor:Stop()
			inst.sg:SetTimeout(20*FRAMES)
        end,

        timeline = 
        {
			TimeEvent(7*FRAMES, function(inst)
				SpawnPrefab("mining_fx").Transform:SetPosition(inst.Transform:GetWorldPosition())
                inst.SoundEmitter:PlaySound("dontstarve_DLC001/common/iceboulder_hit")
				--inst.components.combat:DoAttack(inst.sg.statemem.target)
            end),
            TimeEvent(17*FRAMES, function(inst)
				SpawnPrefab("mining_fx").Transform:SetPosition(inst.Transform:GetWorldPosition())
                inst.SoundEmitter:PlaySound("dontstarve_DLC001/common/iceboulder_hit")
				inst.components.combat:DoAttack(inst.sg.statemem.target)
            end),
			--TimeEvent(27*FRAMES, function(inst)
			--	SpawnPrefab("mining_fx").Transform:SetPosition(inst.Transform:GetWorldPosition())
            --    inst.SoundEmitter:PlaySound("dontstarve_DLC001/common/iceboulder_hit")
			--	inst.components.combat:DoAttack(inst.sg.statemem.target)
            --end),
        },

        ontimeout=function(inst)
			inst.sg:GoToState("jumpin_pre")
		end,
    },
   
	--召唤 短召唤
	State{
        name = "summon",
        tags = { "attack", "busy", "canrotate"},

        onenter = function(inst,tar)
			inst.sg.statemem.target = tar
            inst.components.combat:StartAttack()
			inst.components.locomotor:Stop()
			
            inst.AnimState:PlayAnimation("staff_pre")
            --inst.AnimState:PushAnimation("staff", false)
            inst.components.locomotor:Stop()
			inst.sg:SetTimeout(30*FRAMES)
        end,

        timeline = 
        {
			TimeEvent(0*FRAMES, function(inst)
                inst.SoundEmitter:PlaySound("zg_summon/zg_summon/summon") 
            end),
            TimeEvent(13*FRAMES, function(inst)
				inst.components.combat:DoAttack(inst.sg.statemem.target)
            end),
        },

        ontimeout=function(inst)
			inst.sg:GoToState("idle")
		end,
    },
	
	--召唤2 长召唤
	State{
        name = "summon2",
        tags = { "attack", "busy", "canrotate"},

        onenter = function(inst,tar)
			inst.sg.statemem.target = tar
            inst.components.combat:StartAttack()
			inst.components.locomotor:Stop()
			
            inst.AnimState:PlayAnimation("staff_pre")
            inst.AnimState:PushAnimation("staff", false)
            inst.components.locomotor:Stop()
			inst.sg:SetTimeout(60*FRAMES)
        end,

        timeline = 
        {
			TimeEvent(0*FRAMES, function(inst)
                inst.SoundEmitter:PlaySound("zg_summon/zg_summon/summon2") 
            end),
            TimeEvent(58*FRAMES, function(inst)
				inst.components.combat:DoAttack(inst.sg.statemem.target)
            end),
        },

        ontimeout=function(inst)
			inst.sg:GoToState("idle")
		end,
    },
	
	--读书 书本法术
	State{
        name = "readbook",
        tags = { "attack", "busy", "canrotate"},

        onenter = function(inst,tar)
			inst.sg.statemem.target = tar
            inst.components.combat:StartAttack()
			inst.components.locomotor:Stop()
			
            inst.AnimState:PlayAnimation("action_uniqueitem_pre")
            inst.AnimState:PushAnimation("book", false)
			inst.AnimState:Show("ARM_normal")
			
            inst.components.locomotor:Stop()
			inst.sg:SetTimeout(60*FRAMES)
        end,

        timeline = 
        {
			TimeEvent(0 * FRAMES, function(inst)
                inst.SoundEmitter:PlaySound("zg_summon/zg_summon/summon2")
				
				local fx = SpawnPrefab("book_fx")
                fx.entity:SetParent(inst.entity)
                fx.Transform:SetPosition(0, 0.2, 0)
                inst.sg.statemem.book_fx = fx
            end),	
            TimeEvent(58*FRAMES, function(inst)
				inst.components.combat:DoAttack(inst.sg.statemem.target)
            end),
        },

        ontimeout=function(inst)
			inst.sg:GoToState("idle")
		end,
		
		onexit = function(inst)
            if inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS) then
                inst.AnimState:Show("ARM_carry") 
                inst.AnimState:Hide("ARM_normal")
            end
			if inst.sg.statemem.book_fx then
                inst.sg.statemem.book_fx:Remove()
                inst.sg.statemem.book_fx = nil
            end
        end,
    },
   ---------------------------------------------------------------------------------------------------------------
	--移动
    State{
        name = "run_start",
        tags = {"moving", "running", "canrotate"},
        
        onenter = function(inst)
			inst.components.locomotor:RunForward()
            inst.AnimState:PlayAnimation(inst.hasshield and "zg_run_pre" or "run_pre")
            inst.sg.mem.foosteps = 0
        end,
        onupdate = function(inst)
            inst.components.locomotor:RunForward()
        end,
        events=
        {   
            EventHandler("animover", function(inst) inst.sg:GoToState("run") end ),        
        }, 
        timeline=
        {
        
            TimeEvent(4*FRAMES, function(inst)
                PlayFootstep(inst)
                DoFoleySounds(inst)
            end),
        },        
        
    },
    State{ 
        name = "run",
        tags = {"moving", "running", "canrotate"},  
        onenter = function(inst) 
            inst.components.locomotor:RunForward()
            inst.AnimState:PlayAnimation(inst.hasshield and "zg_run_loop" or "run_loop")  
			inst.sg.statemem.startpos=inst:GetPosition()
        end,
        onupdate = function(inst)
            inst.components.locomotor:RunForward()
        end,
        timeline=
        {
            TimeEvent(7*FRAMES, function(inst)
				inst.sg.mem.foosteps = inst.sg.mem.foosteps + 1
                PlayFootstep(inst, inst.sg.mem.foosteps < 5 and 1 or .6)
                DoFoleySounds(inst)
            end),
            TimeEvent(15*FRAMES, function(inst)
				inst.sg.mem.foosteps = inst.sg.mem.foosteps + 1
                PlayFootstep(inst, inst.sg.mem.foosteps < 5 and 1 or .6)
                DoFoleySounds(inst)
            end),
        },       
        events=
        {   
            EventHandler("animover", function(inst) 
				inst.sg.statemem.endpos=inst:GetPosition()
				local dist = inst.sg.statemem.startpos:Dist(inst.sg.statemem.endpos)
				if dist< .5+ math.random()*.25 then
					inst.sg:GoToState("jump") 
				else
					inst.sg:GoToState("run") 
				end
			end ),        
        },      
    }, 
    State{
        name = "run_stop",
        tags = {"canrotate", "idle"},        
        onenter = function(inst) 
            inst.components.locomotor:Stop()
            inst.AnimState:PlayAnimation(inst.hasshield and "zg_run_pst" or "run_pst")
        end,        
        events=
        {   
            EventHandler("animover", function(inst) inst.sg:GoToState("idle") end ),        
        },       
    },  
	
	--跳入
	State{
        name = "jumpin_pre",
        tags = { "attack","busy", "canrotate" },

        onenter = function(inst)
            inst.components.locomotor:Stop()
            inst.AnimState:PlayAnimation("jump_pre", false)
        end,

        events =
        {
            EventHandler("animover", function(inst)
                inst.sg:GoToState("jumpin")
            end),
        },
    },
	State{
        name = "jumpin",
        tags = { "attack","busy", "canrotate" },

        onenter = function(inst)
            inst.components.locomotor:Stop()
            inst.AnimState:PlayAnimation("jump", false)
        end,

        events =
        {
            EventHandler("animover", function(inst)
                inst.sg:GoToState("jumpout")
            end),
        },
		onexit=function(inst)
			--通知in关闭
			if inst.holein then
				inst.holein.sg:GoToState("closing")
			end
			--通知out开启并瞬移到out位置
			if inst.holetarget then
				local holeout=zg_spawn(inst.holetarget, "zg_fx_holeout")
				inst.holeout=holeout
				if inst.holeout then
					inst.holeout.sg:GoToState("opening")
					inst.Transform:SetPosition(inst.holeout.Transform:GetWorldPosition())
				end
			end
			
		end,
    },
	State{
        name = "jumpout",
        tags = { "attack","busy", "canrotate" },

        onenter = function(inst)
            inst.components.locomotor:Stop()
            inst.AnimState:PlayAnimation("jumpout", false)
        end,

        events =
        {
            EventHandler("animover", function(inst)
				local equip = inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS)
				if not equip then inst:EquipRandomWeapon(inst) end
                inst.sg:GoToState("attack")
            end),
        },
    },
	
	
	--武器坏了
	State{
        name = "toolbroke",
        tags = { "busy"},

        onenter = function(inst, tool)
            inst.components.locomotor:StopMoving()
            inst.AnimState:PlayAnimation("hit")
            inst.SoundEmitter:PlaySound("dontstarve/wilson/use_break")
            inst.AnimState:Hide("ARM_carry") 
            inst.AnimState:Show("ARM_normal") 
            SpawnPrefab("brokentool").Transform:SetPosition(inst.Transform:GetWorldPosition())
            --inst.sg.statemem.tool = tool
			inst.sg:SetTimeout(2*FRAMES)
        end,
		
		ontimeout=function(inst)
			inst.sg:GoToState("idle")
		end,

        onexit = function(inst)
            inst:EquipRandomWeapon(inst)
        end,
    },
	
	--被打
	State{
        name = "hit",
        tags = { "busy", "pausepredict" },

        onenter = function(inst)
            
            inst.components.locomotor:Stop()

            inst.AnimState:PlayAnimation(inst.hasshield and "zg_atk_pre" or "hit")

            inst.SoundEmitter:PlaySound("dontstarve/wilson/hit")

            local stun_frames = 6
            
            inst.sg:SetTimeout(stun_frames * FRAMES)
        end,

        ontimeout = function(inst)
            inst.sg:GoToState("idle", true)
        end,
    },

	--被冻住
	State {
		name = "frozen",
		tags = {"busy"},
		
        onenter = function(inst)
            inst.AnimState:PlayAnimation("frozen")
            inst.Physics:Stop()
        end,
    },
	
	--触电
	--[[
	State{
        name = "electrocute",
        tags = { "busy"},

        onenter = function(inst)
            
            inst.components.locomotor:Stop()
            
            inst.fx = SpawnPrefab("shock_fx")
            
            inst.fx.entity:SetParent(inst.entity)
            inst.fx.entity:AddFollower()
            inst.fx.Follower:FollowSymbol(inst.GUID, "swap_shock_fx", 0, 0, 0)

            inst.AnimState:PlayAnimation("shock")
            inst.AnimState:PushAnimation("shock_pst", false)
			
            inst.SoundEmitter:PlaySound(inst.zg_hurtsound)

            inst.sg:SetTimeout(8 * FRAMES + inst.AnimState:GetCurrentAnimationLength())
        end,

        events =
        {
            EventHandler("animover", function(inst)
                if inst.fx ~= nil then
                    inst.fx:Remove()
                    inst.fx = nil
                end
            end),

            EventHandler("animqueueover", function(inst)
                if inst.AnimState:AnimDone() then
                    inst.sg:GoToState("idle")
                end
            end),
        },

        ontimeout = function(inst)
            inst.sg:GoToState("idle", true)
        end,

        onexit = function(inst)
            if inst.fx ~= nil then
                inst.fx:Remove()
                inst.fx = nil
            end
        end,
    },]]
}
CommonStates.AddFrozenStates(states)
    
return StateGraph("sg_manshape", states, events, "idle", actionhandlers)

