--local TheWorld = GLOBAL.TheWorld
local GetTime = GLOBAL.GetTime
local ACTIONS = GLOBAL.ACTIONS
local TUNING = GLOBAL.TUNING
local STRINGS = GLOBAL.STRINGS
local FRAMES = GLOBAL.FRAMES
local require = GLOBAL.require
local IsServer = GLOBAL.TheNet:GetIsServer()
local TheInput = GLOBAL.TheInput
local ThePlayer = GLOBAL.ThePlayer

local easing = require("easing")
local Inv = require "widgets/inventorybar"

GLOBAL.EQUIPSLOT_IDS = {}
local slot = 0
for k, v in pairs(GLOBAL.EQUIPSLOTS) do
    slot = slot + 1
    GLOBAL.EQUIPSLOT_IDS[v] = slot
end
slot = nil

----扩展装备栏
GLOBAL.EQUIPSLOTS =
{
    HANDS = "hands",
    HEAD = "head",
    BODY = "body",
	
    ZGHAND2 = "zghand2",
    ZGAMULET = "zgamulet",
	ZGBACKPACK = "zgbackpack",
	ZGRING1 = "zgring1",
	ZGRING2 = "zgring2",
}
local EQUIPSLOTS = GLOBAL.EQUIPSLOTS

AddComponentPostInit("inventory", function(self, inst)
    local original_Equip = self.Equip
    self.Equip = function(self, item, old_to_active)
        if original_Equip(self, item, old_to_active) and item and item.components and item.components.equippable then
            local eslot = item.components.equippable.equipslot
            if self.equipslots[eslot] ~= item then
                if eslot == EQUIPSLOTS.ZGBACKPACK and item.components.container ~= nil then
                    self.inst:PushEvent("setoverflow", { overflow = item })
                end
            end
            return true
        else
            return
        end
    end

    self.GetOverflowContainer = function()
        if self.ignoreoverflow then
            return
        end
        local item = self:GetEquippedItem(EQUIPSLOTS.ZGBACKPACK)
        return item ~= nil and item.components.container or nil
    end
end)

AddGlobalClassPostConstruct("widgets/inventorybar", "Inv", function()
    local Old_Refresh = Inv.Refresh or function() return "" end
    local Old_Rebuild = Inv.Rebuild or function() return "" end

	----加载额外的物品栏
    function Inv:LoadExtraSlots(self)
        self.bg:SetScale(1.4,1,1.25)
        self.bgcover:SetScale(1.4,1,1.25)

        if self.addextraslots == nil then
            self.addextraslots = 1
			
			self:AddEquipSlot(EQUIPSLOTS.ZGBACKPACK, "images/zg_backpack.xml", "zg_backpack.tex")
			self:AddEquipSlot(EQUIPSLOTS.ZGAMULET, "images/zg_neck.xml", "zg_neck.tex")
			self:AddEquipSlot(EQUIPSLOTS.ZGHAND2, "images/zg_hand2.xml", "zg_hand2.tex")
			
			if self.inspectcontrol then
                local W = 68
                local SEP = 12
                local INTERSEP = 28
                local inventory = self.owner.replica.inventory
                local num_slots = inventory:GetNumSlots()
                local num_equip = #self.equipslotinfo
                local num_buttons = self.controller_build and 0 or 1
                local num_slotintersep = math.ceil(num_slots / 5)
                local num_equipintersep = num_buttons > 0 and 1 or 0
                local total_w = (num_slots + num_equip + num_buttons) * W + (num_slots + num_equip + num_buttons - num_slotintersep - num_equipintersep - 1) * SEP + (num_slotintersep + num_equipintersep) * INTERSEP
            	self.inspectcontrol.icon:SetPosition(-4, 6)
            	self.inspectcontrol:SetPosition((total_w - W) * .5 + 3, -6, 0)
            end
			
        end
    end

	----刷新和重建
    function Inv:Refresh()
        Old_Refresh(self)
        Inv:LoadExtraSlots(self)
    end

    function Inv:Rebuild()
        Old_Rebuild(self)
        Inv:LoadExtraSlots(self)
    end
end)

----为了兼容
AddPrefabPostInit("inventory_classified", function(inst)
    function GetOverflowContainer(inst)
        local item = inst.GetEquippedItem(inst, EQUIPSLOTS.ZGBACKPACK)
        return item ~= nil and item.replica.container or nil
    end

    function Count(item)
        return item.replica.stackable ~= nil and item.replica.stackable:StackSize() or 1
    end

    function Has(inst, prefab, amount)
        local count =
            inst._activeitem ~= nil and
            inst._activeitem.prefab == prefab and
            Count(inst._activeitem) or 0

        if inst._itemspreview ~= nil then
            for i, v in ipairs(inst._items) do
                local item = inst._itemspreview[i]
                if item ~= nil and item.prefab == prefab then
                    count = count + Count(item)
                end
            end
        else
            for i, v in ipairs(inst._items) do
                local item = v:value()
                if item ~= nil and item ~= inst._activeitem and item.prefab == prefab then
                    count = count + Count(item)
                end
            end
        end

        local overflow = GetOverflowContainer(inst)
        if overflow ~= nil then
            local overflowhas, overflowcount = overflow:Has(prefab, amount)
            count = count + overflowcount
        end

        return count >= amount, count
    end

    if not IsServer then
        inst.GetOverflowContainer = GetOverflowContainer
        inst.Has = Has
    end
end)

----修改默认的栏目
function backpackpostinit(inst)
    if GLOBAL.TheNet:GetIsServer() then
        inst.components.equippable.equipslot = GLOBAL.EQUIPSLOTS.ZGBACKPACK or GLOBAL.EQUIPSLOTS.BODY
    end
end

function amuletpostinit(inst)
    if GLOBAL.TheNet:GetIsServer() then
        inst.components.equippable.equipslot = GLOBAL.EQUIPSLOTS.ZGAMULET or GLOBAL.EQUIPSLOTS.BODY
    end
end

AddPrefabPostInit("blueamulet", amuletpostinit)
AddPrefabPostInit("purpleamulet", amuletpostinit)
AddPrefabPostInit("orangeamulet", amuletpostinit)
AddPrefabPostInit("greenamulet", amuletpostinit)
AddPrefabPostInit("yellowamulet", amuletpostinit)

AddPrefabPostInit("backpack", backpackpostinit)
AddPrefabPostInit("krampus_sack", backpackpostinit)
AddPrefabPostInit("piggyback", backpackpostinit)
AddPrefabPostInit("icepack", backpackpostinit)

------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------

----用于修改函数尾的闭包, 不知道我这么说对不对
local function Zg_Mod(cmp,fn_name,fn)
	local old_fn = cmp[fn_name]
	cmp[fn_name] = function(self,...)
		local res = old_fn(self,...)
		return fn(res,self,...)
	end
end

----修改显示名称 
Zg_Mod(GLOBAL.EntityScript, "GetDisplayName", function(res, self)

	----显示血量
	if self.zg_hp_max and self.zg_hp_max ~= 0 and not self:HasTag("playerghost") then

		local cur = math.floor(self.zg_hp_cur+.5)
		local max = math.floor(self.zg_hp_max+.5)
		
		res = res.." < 生命: "..cur.." / "..max.." >"    
		
	end
	
	----枪械耐久
	if self.zg_detail_gunhp and self.zg_detail_gunhp ~= 0 then
		local durable = self.zg_detail_gunhp
		res = res.."\n".." ( 真实耐久: "..durable.." )"
	end
	
---------------------------------------------------------------------------------------------------
	
	----显示盾牌属性
	if self:HasTag("zg_shield") and self["zg_shield_block"] and self["zg_shield_absorb"] and self["zg_shield_absorb2"] and self["zg_shield_destill"] and 
		self.zg_detail_condition and self.zg_detail_condition ~= 0
	then
	
		--格挡率
		local block = math.floor(self["zg_shield_block"] * 100)
		res = res.."\n".." ( 法向格挡率: "..block.."% )"
		
		--格挡成功时伤害吸收
		local absorb = math.floor(self["zg_shield_absorb"] * 100)
		res = res.."\n".." ( 格挡成功时伤害吸收: "..absorb.."% )"
		
		--格挡成功时伤害吸收
		local absorb2 = math.floor(self["zg_shield_absorb2"] * 100)
		res = res.."\n".." ( 格挡失败时伤害吸收: "..absorb2.."% )"
	
		--降低僵直时间
		local destill = math.floor(self["zg_shield_destill"] * 100)
		res = res.."\n".." ( 降低僵直时间: "..destill.."% )"
	
		local condition = self.zg_detail_condition
		res = res.."\n".."( 耐久: "..condition.." )"
	end
---------------------------------------------------------------------------------------------------
	
	----显示武器属性
	if self.zg_detail_dmg and self.zg_detail_dmg ~= 0 and 
		self.zg_detail_useleft and self.zg_detail_useleft ~= 0
	then
	
		local dmg = math.floor(self.zg_detail_dmg * 10 + .5) / 10
		local useleft = self.zg_detail_useleft
		
		res = res.."\n".." ( 伤害: "..dmg.." , 剩余耐久: "..useleft.." )"
		
	end
	
	----显示非耐久武器
	if self.zg_detail_dmg and self.zg_detail_dmg ~= 0 and 
		self.zg_detail_useleft and self.zg_detail_useleft == 0
	then
		local dmg = math.floor(self.zg_detail_dmg * 10 + .5) / 10
		res = res.."\n".." ( 伤害: "..dmg.." )"
	end
	
	----显示非武器耐久
	if self.zg_detail_useleft and self.zg_detail_useleft ~= 0 and 
		self.zg_detail_dmg and self.zg_detail_dmg == 0
	then
		local useleft = self.zg_detail_useleft
		res = res.."\n".." ( 剩余耐久: "..useleft.." )"
	end
	
	----显示防具耐久
	if self.zg_detail_absorb and self.zg_detail_absorb ~= 0 and 
		self.zg_detail_condition and self.zg_detail_condition ~= 0
	then
		local absorb = self.zg_detail_absorb * 100
		absorb = math.floor(absorb * 10+ .5)/10
		local condition = self.zg_detail_condition
		res = res.."\n".."( 吸收物理伤害: "..absorb.." % , ".." 耐久: "..condition.." )"
	end

-----------------------------------------------------------------------------------------------------	
	----显示装备层防护属性
	local stimulitable00 = 
	{
		fire = "装备层防火系", 
		ice = "装备层防冰系", 
		electric = "装备层防电系", 
		stone = "装备层防土系", 
		wood = "装备层防木系",	
		dark = "装备层防暗系", 
		ray = "装备层防光系",  
		bone = "装备层防魔法", 
		bullet = "装备层防子弹", 
		physics = "装备层防物理",		--物理系包括子弹等, 属于大类
	}
	for k, v in pairs(stimulitable00) do
		if self["zg_equ_anti_"..k] and self["zg_equ_anti_"..k] > 0 then
			local percent = math.floor(self["zg_equ_anti_"..k] * 100)
			res = res.."\n".."( "..v..": "..percent.."% )"
		end
	end
	----显示属性增加与减益
	local stimulitable01 = 
	{
		fire = "火系抗性", 
		ice = "冰系抗性", 
		electric = "电系抗性", 
		stone = "土系抗性", 
		wood = "木系抗性",	
		dark = "暗系抗性", 
		ray = "光系抗性",  
		bone = "魔法抗性", 
		bullet = "子弹抗性", 
		physics = "物理抗性",		--物理系包括子弹等, 属于大类
	}
	for k, v in pairs(stimulitable01) do
		if self["zg_add_anti_"..k] and self["zg_add_anti_"..k] ~= 0 then
			local percent = math.floor(self["zg_add_anti_"..k] * 100)
			local sign = percent > 0 and "+" or ""
			res = res.."\n".."[ "..v..": "..sign..percent.."% ]"
		end
	end
---------------------------------------------------------------------------------------------------
	
	----食物属性
	if ( self.zg_detail_hungervalue and self.zg_detail_hungervalue ~= 0 ) or
	   ( self.zg_detail_healthvalue and self.zg_detail_healthvalue ~= 0 ) or
	   ( self.zg_detail_sanityvalue and self.zg_detail_sanityvalue ~= 0 ) 
	then
		local _hungervalue = math.floor(self.zg_detail_hungervalue * 10 + .5) / 10
		local _healthvalue = math.floor(self.zg_detail_healthvalue * 10 + .5) / 10
		local _sanityvalue = math.floor(self.zg_detail_sanityvalue * 10 + .5) / 10
		
		res = res.."\n".." ( 食物属性饱食: ".._hungervalue.." / 回血: ".._healthvalue.." / 理智: ".._sanityvalue.." )"
	end
	
	----治疗属性
	if self.zg_detail_healvalue and self.zg_detail_healvalue ~= 0 then
		local _healvalue = math.floor(self.zg_detail_healvalue * 10 + 0.5) / 10
		
		res = res.."\n".." ( 治疗: ".._healvalue.." )"
	end
	
	----隔热属性
	if self.zg_detail_insulator and self.zg_detail_insulator ~= 0 then
		local _insulator = self.zg_detail_insulator
		
		res = res.."\n".." ( 隔热: ".._insulator.." 秒 )"
	end
	
	----重量
	if self.zg_detail_heavy and self.zg_detail_heavy ~= 0 then
		local _heavy = math.floor(self.zg_detail_heavy * 10 + 0.5) / 10
		
		res = res.."\n".." ( ".._heavy.." kg/one )"
	end
	
	return res
	
end)

----为玩家带上物品属性
AddPrefabPostInitAny(function(inst)
	if inst then
----------------------------------------------附加属性显示, 主要是针对变动后--------------------------------
	
		----整型
		local detail_ushortint = 
		{	
			"zg_hp_cur", "zg_hp_max",
			
			"zg_detail_useleft",
			
			"zg_detail_condition", 
			
			"zg_detail_insulator",
	
			--"zg_detail_gunhp", 
		}
		for k, v in pairs(detail_ushortint) do
			inst[v] = 0
			inst["net_"..v] = GLOBAL.net_ushortint( inst.GUID, v, v.."_dirty" )
				inst:ListenForEvent( v.."_dirty", function()
				inst[v] = inst["net_"..v]:value()
			end)
		end
		
		----浮点型
		local detail_float = 
		{
			
			----------------
			----基础辅助显示
			"zg_detail_dmg", 
			
			"zg_detail_absorb",
			
			"zg_detail_hungervalue", "zg_detail_healthvalue", "zg_detail_sanityvalue", 
			
			"zg_detail_healvalue",
			
			"zg_detail_heavy",
			
		}
		for k, v in pairs(detail_float) do
			inst[v] = 0
			inst["net_"..v] = GLOBAL.net_float( inst.GUID, v, v.."_dirty" )
				inst:ListenForEvent( v.."_dirty", function()
				inst[v] = inst["net_"..v]:value()
			end)
		end
------------------------------------------------------------------------------------------------------------------------		
		----整型新体系
		local detail_ushortint1 = 
		{	
			"zg_detail_gunhp", 
		}
		for k, v in pairs(detail_ushortint1) do
			inst["net_"..v] = GLOBAL.net_ushortint( inst.GUID, v, v.."_dirty" )
				inst:ListenForEvent( v.."_dirty", function()
				inst[v] = inst["net_"..v]:value()
			end)
		end
		
		----浮点型新体系
		local detail_float1 = 
		{
			----盾牌的三个基础属性
			"zg_shield_destill", "zg_shield_block", "zg_shield_absorb",
			
			----抗性加成
			"zg_add_anti_fire", "zg_add_anti_ice", "zg_add_anti_electric", "zg_add_anti_stone", "zg_add_anti_wood", "zg_add_anti_ray", 
			"zg_add_anti_dark",
			"zg_add_anti_bone",
			"zg_add_anti_bullet",
			
			----装备层抗性
			"zg_equ_anti_fire", "zg_equ_anti_ice", "zg_equ_anti_electric", "zg_equ_anti_stone", "zg_equ_anti_wood", "zg_equ_anti_ray", 
			"zg_equ_anti_dark",
			"zg_equ_anti_bone",
			"zg_equ_anti_bullet",
			
		}
		for k, v in pairs(detail_float1) do
			inst["net_"..v] = GLOBAL.net_float( inst.GUID, v, v.."_dirty" )
				inst:ListenForEvent( v.."_dirty", function()
				inst[v] = inst["net_"..v]:value()
			end)
		end
	
------------------------------------------------------------------------------------------------------------	
	
		if GLOBAL.TheNet:GetIsServer() then
		
			----对于新属性的设置
			local shuxing_table = 
			{
				----盾牌的三个基础属性
				"zg_shield_destill", "zg_shield_block", "zg_shield_absorb",
				
				----抗性加成
				"zg_add_anti_fire", "zg_add_anti_ice", "zg_add_anti_electric", "zg_add_anti_stone", "zg_add_anti_wood", "zg_add_anti_ray", 
				"zg_add_anti_dark",
				"zg_add_anti_bone",
				"zg_add_anti_bullet",
				
				----装备层抗性
				"zg_equ_anti_fire", "zg_equ_anti_ice", "zg_equ_anti_electric", "zg_equ_anti_stone", "zg_equ_anti_wood", "zg_equ_anti_ray", 
				"zg_equ_anti_dark",
				"zg_equ_anti_bone",
				"zg_equ_anti_bullet",
				
				----枪械耐久
				"zg_detail_gunhp", 
			}
			for k, v in pairs(shuxing_table) do
				if inst["net"..v] and inst[v] then
					inst["net"..v]:set(inst[v])
				end
			end
			
			----	血量模块
			if inst.components.health then
				inst.net_zg_hp_cur:set(inst.components.health.currenthealth)
				inst.net_zg_hp_max:set(inst.components.health.maxhealth)
			end
			
			----	武器属性
			if inst.components.weapon then
				inst.net_zg_detail_dmg:set(inst.components.weapon.damage)		
			end
			
			----	耐久
			if inst.components.finiteuses then
				inst.net_zg_detail_useleft:set(inst.components.finiteuses.current)
			end
			
			----	防具属性
			if inst.components.armor then
				inst.net_zg_detail_absorb:set(inst.components.armor.absorb_percent)
				inst.net_zg_detail_condition:set(inst.components.armor.condition)		
			end
			
			----	保温属性
			if inst.components.insulator then
				inst.net_zg_detail_insulator:set( inst.components.insulator.insulation )
			end
		
			----	治疗属性
			if inst.components.healer then
				inst.net_zg_detail_healvalue:set( inst.components.healer.health )
			end
			
			----	食物属性
			if inst.components.edible then
				inst.net_zg_detail_hungervalue:set( inst.components.edible:GetHunger() )
				inst.net_zg_detail_healthvalue:set( inst.components.edible:GetHealth() )
				inst.net_zg_detail_sanityvalue:set( inst.components.edible:GetSanity() )
			end
		
		end
	end
end)

----	血量模块
AddComponentPostInit("health", function(cmp)

	function cmp:DoDelta(amount, overtime, cause, ignore_invincible, afflicter, ignore_absorb)
		if self.redirect ~= nil and self.redirect(self.inst, amount, overtime, cause, ignore_invincible, afflicter, ignore_absorb) then
			return
		elseif not ignore_invincible and (self.invincible or self.inst.is_teleporting == true) then
			return
		elseif amount < 0 and not ignore_absorb then
			--玩家不使用absorb
			amount = amount - amount * (self.inst:HasTag("player") and 0 or self.absorb)
			
			if afflicter ~= nil and afflicter:HasTag("player") then
				amount = amount - amount * self.playerabsorb
			end
		end

		local old_percent = self:GetPercent()
		self:SetVal(self.currenthealth + amount, cause, afflicter)
		local new_percent = self:GetPercent()

		self.inst:PushEvent("healthdelta", { oldpercent = old_percent, newpercent = self:GetPercent(), overtime = overtime, cause = cause, afflicter = afflicter, amount = amount })

		if self.ondelta ~= nil then
			self.ondelta(self.inst, old_percent, self:GetPercent())
		end
		
		----伤害显示
		amount=self.maxhealth*(new_percent-old_percent)
		if (amount<=-0.1 or amount>=-0.1) and not self.inst:HasTag("zg_no_dmgid") and not ignore_invincible and not overtime then
			local DMGID = GLOBAL.SpawnPrefab("zg_fx_dmgid")
			DMGID.Transform:SetPosition(self.inst.Transform:GetWorldPosition())
			--DMGID.zg_hp_si:set_local(false)
			DMGID.zg_hp_si:set(amount > 0)
			--DMGID.zg_hp_dt:set_local(0)
			DMGID.zg_hp_dt:set(math.abs(amount))
			
			
		end
		
		----显示血量
		if self.inst.zg_hp_cur ~= nil and self.inst.zg_hp_max ~= nil then
			self.inst.net_zg_hp_cur:set(self.currenthealth)
			self.inst.net_zg_hp_max:set(self.maxhealth)
		end
	end

end)

----	武器属性+耐久
AddComponentPostInit("weapon", function(cmp)

	function cmp:SetDamage(dmg)
		self.damage = dmg
		if self.inst.zg_detail_dmg then
			self.inst.net_zg_detail_dmg:set(self.damage)
		end
	end
	
end)
	
AddComponentPostInit("finiteuses", function(cmp)

	function cmp:SetUses(val)
		local was_positive = self.current > 0
		self.current = val
		self.inst:PushEvent("percentusedchange", {percent = self:GetPercent()})
		
		----显示武器耐久
		if self.inst.zg_detail_useleft then
			self.inst.net_zg_detail_useleft:set(self.current)
		end
		
		if self.current <= 0 then
			self.current = 0
			if was_positive and self.onfinished ~= nil then
				self.onfinished(self.inst)
			end
		end
	end
	
end)

----	防具属性
AddComponentPostInit("armor", function(cmp)

	function cmp:InitCondition(amount, absorb_percent)
		self.condition = amount
		self.absorb_percent = absorb_percent
		self.maxcondition = amount
		----显示装备属性
		if self.inst.zg_detail_absorb and self.inst.zg_detail_condition then
			self.inst.net_zg_detail_absorb:set(self.absorb_percent)
			self.inst.net_zg_detail_condition:set(self.condition)
		end
	end
	
	function cmp:SetCondition(amount)
		self.condition = math.min(amount, self.maxcondition)
		self.inst:PushEvent("percentusedchange", { percent = self:GetPercent() })
		
		----显示装备属性
		if self.inst.zg_detail_absorb and self.inst.zg_detail_condition then
			self.inst.net_zg_detail_absorb:set(self.absorb_percent)
			self.inst.net_zg_detail_condition:set(self.condition)
		end

		if self.condition <= 0 then
			self.condition = 0
			--ProfileStatsSet("armor_broke_"..self.inst.prefab, true)
			--ProfileStatsSet("armor", self.inst.prefab)

			if self.onfinished ~= nil then
				self.onfinished()
			end

			self.inst:Remove()
		end
	end
	
	function cmp:SetAbsorption(absorb_percent)
		self.absorb_percent = absorb_percent
		
		----显示装备属性
		if self.inst.zg_detail_absorb and self.inst.zg_detail_condition then
			self.inst.net_zg_detail_absorb:set(self.absorb_percent)
			self.inst.net_zg_detail_condition:set(self.condition)
		end
	end

end)

----	隔热属性
AddComponentPostInit("insulator", function(cmp)

	function cmp:SetInsulation(val)
		self.insulation = val	
		
		----隔热属性
		if self.inst.zg_detail_insulator then
			self.inst.net_zg_detail_insulator:set(self.insulation)
		end
	end

end)

----	治疗属性
AddComponentPostInit("healer", function(cmp)

	function cmp:SetHealthAmount(health)
		self.health = health
		
		----治疗属性
		if self.inst.zg_detail_healvalue then
			self.inst.net_zg_detail_healvalue:set(self.health)
		end
	end
	
end)

----	食物属性
AddComponentPostInit("perishable", function(cmp)

	local function Update(inst, dt)
		if inst.components.perishable then
			
			local modifier = 1
			local owner = inst.components.inventoryitem and inst.components.inventoryitem.owner or nil
			if not owner and inst.components.occupier then
				owner = inst.components.occupier:GetOwner()
			end

			if owner then
				if owner:HasTag("fridge") then
					if inst:HasTag("frozen") and not owner:HasTag("nocool") and not owner:HasTag("lowcool") then
						modifier = TUNING.PERISH_COLD_FROZEN_MULT
					else
						modifier = TUNING.PERISH_FRIDGE_MULT
					end
				elseif owner:HasTag("spoiler") then
					modifier = TUNING.PERISH_GROUND_MULT 
				elseif owner:HasTag("cage") and inst:HasTag("small_livestock") then
					modifier = TUNING.PERISH_CAGE_MULT
				end
			else
				modifier = TUNING.PERISH_GROUND_MULT 
			end

			if inst:GetIsWet() then
				modifier = modifier * TUNING.PERISH_WET_MULT
			end

			
			if GLOBAL.TheWorld.state.temperature < 0 then
				if inst:HasTag("frozen") and not inst.components.perishable.frozenfiremult then
					modifier = TUNING.PERISH_COLD_FROZEN_MULT
				else
					modifier = modifier * TUNING.PERISH_WINTER_MULT
				end
			end

			if inst.components.perishable.frozenfiremult then
				modifier = modifier * TUNING.PERISH_FROZEN_FIRE_MULT
			end

			if GLOBAL.TheWorld.state.temperature > TUNING.OVERHEAT_TEMP then
				modifier = modifier * TUNING.PERISH_SUMMER_MULT
			end

			modifier = modifier * inst.components.perishable.localPerishMultiplyer

			modifier = modifier * TUNING.PERISH_GLOBAL_MULT
			
			local old_val = inst.components.perishable.perishremainingtime
			local delta = dt or (10 + math.random()*FRAMES*8)
			if inst.components.perishable.perishremainingtime then 
				inst.components.perishable.perishremainingtime = inst.components.perishable.perishremainingtime - delta*modifier
				if math.floor(old_val*100) ~= math.floor(inst.components.perishable.perishremainingtime*100) then
					inst:PushEvent("perishchange", {percent = inst.components.perishable:GetPercent()})
				end
			end
			
			----食物属性
			if inst.components.edible then
				if inst.zg_detail_hungervalue and inst.zg_detail_healthvalue and inst.zg_detail_sanityvalue then
					inst.net_zg_detail_hungervalue:set( inst.components.edible:GetHunger() )
					inst.net_zg_detail_healthvalue:set( inst.components.edible:GetHealth() )
					inst.net_zg_detail_sanityvalue:set( inst.components.edible:GetSanity() )
				end
			end

			-- Cool off hot foods over time (faster if in a fridge)
			if inst.components.edible and inst.components.edible.temperaturedelta and inst.components.edible.temperaturedelta > 0 then
				if owner and owner:HasTag("fridge") then
					if not owner:HasTag("nocool") then
						inst.components.edible.temperatureduration = inst.components.edible.temperatureduration - 1
					end
				elseif GLOBAL.TheWorld.state.temperature < TUNING.OVERHEAT_TEMP - 5 then
					inst.components.edible.temperatureduration = inst.components.edible.temperatureduration - .25
				end
				if inst.components.edible.temperatureduration < 0 then inst.components.edible.temperatureduration = 0 end
			end
			
			--trigger the next callback
			if inst.components.perishable.perishremainingtime and inst.components.perishable.perishremainingtime <= 0 then
				inst.components.perishable:Perish()
			end
		end
	end

	function cmp:LongUpdate(dt)
		if self.updatetask ~= nil then
			Update(self.inst, dt or 0)
		end
	end

	function cmp:StartPerishing()
		if self.updatetask ~= nil then
			self.updatetask:Cancel()
			self.updatetask = nil
		end

		local dt = 10 + math.random() * GLOBAL.FRAMES * 8
		self.updatetask = self.inst:DoPeriodicTask(dt, Update, math.random()*2, dt)
	end


end)


------------------------------------------------------------
--------------------------------------------------------------------------------------------
----  	对官方物品加标签

--boss级别 暗金
local function bossthingfn(inst)
	if GLOBAL.TheNet:GetIsServer() then
		inst:AddTag("zg_magic_bossthing")
	end
end 
local bossthingtable = 
{
	--季节boss或领域boss的东西
	"minotaurhorn", "deerclops_eyeball", "bearger_fur", "dragon_scale", 
	"minotaur", "deerclops", "bearger", "dragonfly",
	
}
for k,v in pairs(bossthingtable) do
	AddPrefabPostInit(v, bossthingfn)
end

--次boss级别 亮金
local function debossthingfn(inst)
	if GLOBAL.TheNet:GetIsServer() then
		inst:AddTag("zg_magic_debossthing")
	end
end 
local debossthingtable = 
{	
	--人形兵器或海贼系列
	"zg_ch_wilson", "zg_ch_willow", "zg_ch_wolfgang", "zg_ch_wendy", "zg_ch_wx78", "zg_ch_wicker", "zg_ch_woodie", "zg_ch_wes", "zg_ch_webber", "zg_ch_waxwell", "zg_ch_wigfrid",
	"zg_ch1_zoro", "zg_ch1_luffy", "zg_ch1_sanji", "zg_ch1_buggy",
	
	"zg_ch2_wilson","zg_ch2_willow","zg_ch2_wolfgang_mighty","zg_ch2_wendy","zg_ch2_wx78","zg_ch2_wickerbottom",
	"zg_ch2_woodie","zg_ch2_wes","zg_ch2_waxwell","zg_ch2_wathgrithr","zg_ch2_webber",
}
for k,v in pairs(debossthingtable) do
	AddPrefabPostInit(v, debossthingfn)
end

--稀有 青色
local function rarethingfn(inst)
	if GLOBAL.TheNet:GetIsServer() then
		inst:AddTag("zg_magic_rarething")
	end
end 
local rarethingtable = 
{	
	
	--格罗姆及其物品
	"glommer", "glommerwings", "glommerflower",
	
	--切斯特
	"chester", "chester_eyebone", "hutch", "hutch_fishbowl",
	
}
for k,v in pairs(rarethingtable) do
	AddPrefabPostInit(v, rarethingfn)
end

----	物品文字
local function RGB(r, g, b)
    return { r / 255, g / 255, b / 255, 1 }
end
local zise = RGB(255, 0, 255)
local qingse = RGB(0, 255, 255)
local anjin = RGB(205, 191, 121)
local liangjin = RGB(222, 222, 99)

local ItemTile = GLOBAL.require "widgets/itemtile"
function ItemTile:UpdateTooltip()
    local str = self:GetDescriptionString()
    self:SetTooltip(str)
	
	--Boss级别:暗金
	if self.item:HasTag("zg_magic_bossthing") then
		local colour = anjin
		self:SetTooltipColour( GLOBAL.unpack( colour ) )
		
	--次Boss级别:亮金
	elseif self.item:HasTag("zg_magic_debossthing") then
		local colour = liangjin
		self:SetTooltipColour( GLOBAL.unpack( colour ) )
		
	--稀有装备:青色
	elseif self.item:HasTag("zg_magic_rarething") then
		local colour = qingse
		self:SetTooltipColour( GLOBAL.unpack( colour ) )
	
	--魔法材料:紫色
	elseif self.item:HasTag("zg_magic_material") then
		local colour = zise
		self:SetTooltipColour( GLOBAL.unpack( colour ) )
	
	--其他物品
	else
		if self.item:GetIsWet() then
			self:SetTooltipColour( GLOBAL.unpack( GLOBAL.WET_TEXT_COLOUR ) )
		else
			self:SetTooltipColour( GLOBAL.unpack( GLOBAL.NORMAL_TEXT_COLOUR ) )
		end
	end
end

local HoverText = GLOBAL.require "widgets/hoverer"
function HoverText:OnUpdate()
        
    local using_mouse = self.owner.components and self.owner.components.playercontroller:UsingMouse()        
    
    if using_mouse ~= self.shown then
        if using_mouse then
            self:Show()
        else
            self:Hide()
        end
    end
    
    if not self.shown then 
        return 
    end
    
    local str = nil
    local colour = nil
    if self.isFE == false then 
        str = self.owner.HUD.controls:GetTooltip() or self.owner.components.playercontroller:GetHoverTextOverride()
        
        local tooltip_pos = self.owner.HUD.controls:GetTooltipPos()
        if tooltip_pos then
            self.text:SetPosition(tooltip_pos)
        else
            self.text:SetPosition(self.default_text_pos)
        end

        if self.owner.HUD.controls:GetTooltip() then
            colour = self.owner.HUD.controls:GetTooltipColour()
        end
    else
        str = self.owner:GetTooltip()
        tooltip_pos = self.owner:GetTooltipPos()
        if tooltip_pos then
            self.text:SetPosition(tooltip_pos)
        else
            self.text:SetPosition(self.default_text_pos)
        end
    end

    local secondarystr = nil
 
    local lmb = nil
    if not str and self.isFE == false and self.owner:IsActionsVisible() then

        lmb = self.owner.components.playercontroller:GetLeftMouseAction()
        if lmb then
            
            str = lmb:GetActionString()

            if not clour and lmb.target then
			
                --colour = (lmb.target and lmb.target:GetIsWet()) and GLOBAL.WET_TEXT_COLOUR or GLOBAL.NORMAL_TEXT_COLOUR
				colour = 
					
						--boss级别:暗金
						(lmb.target and lmb.target:HasTag("zg_magic_bossthing")) and anjin or
						
						--次boss级别:亮金
						(lmb.target and lmb.target:HasTag("zg_magic_debossthing")) and liangjin or
						
						--稀有装备:青色
						(lmb.target and lmb.target:HasTag("zg_magic_rarething")) and qingse or
					
						--新世界魔法材料:紫色
						(lmb.target and lmb.target:HasTag("zg_magic_material")) and zise or
						
						--潮湿:蓝色
						(lmb.target and lmb.target:GetIsWet()) and GLOBAL.WET_TEXT_COLOUR or 
						
						--其它白色
						GLOBAL.NORMAL_TEXT_COLOUR

			elseif not clour and lmb.invobject then
				colour = 
					
						--boss级别:暗金
						(lmb.invobject and lmb.invobject:HasTag("zg_magic_bossthing")) and anjin or
						
						--次boss级别:亮金
						(lmb.invobject and lmb.invobject:HasTag("zg_magic_debossthing")) and liangjin or
						
						--稀有装备:青色
						(lmb.invobject and lmb.invobject:HasTag("zg_magic_rarething")) and qingse or
					
						--新世界魔法材料:紫色
						(lmb.invobject and lmb.invobject:HasTag("zg_magic_material")) and zise or			
						
						--潮湿:蓝色
						(lmb.invobject and lmb.invobject:GetIsWet()) and GLOBAL.WET_TEXT_COLOUR or 
						
						--其它白色
						GLOBAL.NORMAL_TEXT_COLOUR
			end
            
            if lmb.target and lmb.target ~= lmb.doer then
                local name = lmb.target:GetDisplayName() or (lmb.target.components.named and lmb.target.components.named.name)
                
                if name then
                    local adjective = lmb.target:GetAdjective()
                    
                    if adjective then
                        str = str.. " " .. adjective .. " " .. name
                    else
                        str = str.. " " .. name
                    end
                    
                    if lmb.target.replica.stackable ~= nil and lmb.target.replica.stackable:IsStack() then
                        str = str .. " x" .. tostring(lmb.target.replica.stackable:StackSize())
                    end
                    if lmb.target.components.inspectable and lmb.target.components.inspectable.recordview and lmb.target.prefab then
                        GLOBAL.ProfileStatsSet(lmb.target.prefab .. "_seen", true)
                    end
                end
            end
        end
        local rmb = self.owner.components.playercontroller:GetRightMouseAction()
        if rmb then
            secondarystr = STRINGS.RMB .. ": " .. rmb:GetActionString()
        end
    end

    if str then
	if self.str ~= self.lastStr then

	    self.lastStr = self.str
    	    self.strFrames = 0

        else
	    self.strFrames = self.strFrames - 1
            if self.strFrames <= 0 then
                if lmb and lmb.target and lmb.target:HasTag("player") then
                    self.text:SetColour(GLOBAL.unpack(lmb.target.playercolour))
                else
                    self.text:SetColour(GLOBAL.unpack(colour or GLOBAL.NORMAL_TEXT_COLOUR))
                end
                self.text:SetString(str)
                self.text:Show()
            end
        end
    else
        self.text:Hide()
    end
    if secondarystr then
        self.secondarytext:SetString(secondarystr)
        self.secondarytext:Show()
    else
        self.secondarytext:Hide()
    end

    local changed = (self.str ~= str) or (self.secondarystr ~= secondarystr)
    self.str = str
    self.secondarystr = secondarystr
    if changed then
        local pos = GLOBAL.TheInput:GetScreenPosition()
        self:UpdatePosition(pos.x, pos.y)
    end
end
----------------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------




